//============================================================================
//
//    Java Text Editor library
//
//  HtmlPanel.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.net.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.*;

import static text.editor.Kit.*;


public class HtmlPanel extends JPanel
                           implements ISyncCloseable
{
    private JEditorPane  text = null;

    public JButton backBtn   = null;
    public JButton nextBtn   = null;
    private JTextField    urlText  = null;

    private String       body = null;

    public boolean hasError = false;

    ArrayList<String> urlList = new ArrayList<String>();
    int currIndex = -1;

    public HtmlPanel(URL url)
    {
        super();

        createPanel();

        setPage(url);
    }

    public String getPanelTitle()
    {
        if (currIndex != -1)
            return "";
        return  urlList.get(currIndex);
    }

    protected void createPanel()
    {
        JToolBar toolBar = new JToolBar();
        toolBar.setPreferredSize(new Dimension(700, 36));
        backBtn  = new JButton (getIconImage("BrowserBefore.GIF"));
        backBtn.setToolTipText("戻る");

        nextBtn  = new JButton (getIconImage("BrowserAfter.GIF"));
        nextBtn.setToolTipText("進む");

        HistoryText h3 = new  HistoryText(32);
        h3.loadHistory("HtmlDialog.url");
        urlText  = h3.text;
        toolBar.add(backBtn);
        toolBar.add(nextBtn);
        toolBar.add(h3);

        text = new JEditorPane();
        text.setFont(EditorConfig.font);
        text.setEditable(false);
        JScrollPane scroll = new JScrollPane(text);

        setLayout(new BorderLayout());
        add(toolBar, BorderLayout.NORTH);
        add(scroll);

        text.addHyperlinkListener(new HtmllinkListener()); 

        backBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               backTo();
            }
        });
        nextBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               forwardTo();
            }
        });
        urlText.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               showInputUrl();
            }
        });

    }

    private void addList(String s)
    {
        urlText.setText(s); 
        if (!s.isEmpty())
        {
            urlList.add(s);
            currIndex = urlList.size() - 1;
        }
    }

    public void setPage(java.net.URL linkURL)
    {
        try
        {
            text.setContentType("text/html; charset=UTF-8 ");
            text.setPage(linkURL);
        } 
        catch (Exception ex) 
        {
            hasError = true;
            text.setText(ex.toString());
        }
        finally
        {
            if (!hasError)
            {
                addList(linkURL.toString());
            }
        }
    }

    public void backTo()
    {
        if (currIndex == -1)
            return;
        if (currIndex > 0)
        {
            --currIndex;
            showCurrent();
        }
    }

    public void forwardTo()
    {
        if (currIndex == -1)
            return;
        if (currIndex < urlList.size() - 1)
        {
            ++currIndex;
            showCurrent();
        }
    }

    private void showInputUrl()
    {
        try
        {
            String s = urlText.getText();
            if (!s.isEmpty())
            {
                text.setContentType("text/html; charset=UTF-8 ");
                text.setPage(s);
                addList(s); 
            }
        }
        catch(Exception e){}
        
    }

    private void showCurrent()
    {
        try
        {
            String s = urlList.get(currIndex);
            text.setContentType("text/html; charset=UTF-8 ");
            text.setPage(s);
            urlText.setText(s); 
        }
        catch(Exception ex){ ex.printStackTrace(); }
    }

    public boolean disposed = false;

    public boolean tabClose()
    {
        return true;
    }

    class HtmllinkListener implements HyperlinkListener
    {
        protected Cursor oldCursor;

        public void hyperlinkUpdate(HyperlinkEvent e)
        {
            {
                HyperlinkEvent.EventType type = e.getEventType();
                if (type == HyperlinkEvent.EventType.ENTERED)
                {
                    oldCursor = getCursor();
                    text.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

                }
                else if (type == HyperlinkEvent.EventType.EXITED)
                {
                    text.setCursor(oldCursor);
                }
                else if (type == HyperlinkEvent.EventType.ACTIVATED)
                {
                    if (e instanceof HTMLFrameHyperlinkEvent)
                    {
                         HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
                         HTMLDocument doc = (HTMLDocument)text.getDocument();
                         doc.processHTMLFrameHyperlinkEvent(evt);
                         addList(e.getURL().toString());
                    }
                    else
                    {
                        setPage(e.getURL());
                    }
                }
            }
        }
     }
}



