//============================================================================
//
//    Java Text Editor library
//
//    Kit.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;

public class Kit
{
    public static String HOME; 
    public static JFrame mainFrame = null;

    private static String selFile = null;

    public static void setMainFrame(JFrame mf)
    {
        mainFrame = mf;
    }

    public static JFrame getMainFrame()
    {
        return mainFrame;
    }

    public static void initialize(String homepath)
    {
        HOME = homepath;
        EditorConfig.initialize();
        HighLightControler.initialize();
        UserHistory.initilaize();
    }

    private static CustomTabPane  middleTab  = null;

    public static void setCostomTab(CustomTabPane tabpane)
    {
        middleTab = tabpane;
    }

    public static void addTab(final String title, final JComponent c)
    {
        if (middleTab == null)
            return;

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                addTabSub(title, c);
            }
        });
    }

    private static void addTabSub(final String title, final JComponent c)
    {
        int idx = middleTab.getTabCount();
        middleTab.addTab(title, c);
        middleTab.setSelectedIndex(idx);
        c.requestFocus();
    }

    public static void dockReleaseTab( final JComponent c)
    {
        if (middleTab == null)
            return;

        int idx = middleTab.indexOfComponent(c);
        if (idx == -1)
            return;

        TabOfTab tt = (TabOfTab)middleTab.getTabComponentAt(idx);
        middleTab.removeTabAt(idx);
        final String title = tt.label.getText();
        final JFrame fm = new JFrame(title);
        if (getMainFrame() != null)
            fm.setIconImage(getMainFrame().getIconImage());

        fm.getContentPane().add(c);
        fm.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                dockTab(fm, c, title);
            }
        });

        Point pt = new Point(0,0);
        SwingUtilities.convertPointToScreen(pt, middleTab);
        fm.setSize(middleTab.getSize());
        fm.setLocation(pt);
        fm.setVisible(true);

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                fm.requestFocus();
            }
        });
    }

    public static void dockTab(JFrame fm, JComponent s, String title)
    {
        if (middleTab == null)
            return;

        fm.getContentPane().remove(s);

        addTab(title, s);
    }

    public static void removeTab(final JComponent c)
    {
        if (middleTab == null)
            return;

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                removeTabSub(c);
            }
        });
    }

    private static void removeTabSub(final JComponent c)
    {
        int idx = middleTab.indexOfComponent(c);
        if (idx != -1)
        {
            ISyncCloseable s = (ISyncCloseable)c;
            if (s.tabClose())
                middleTab.removeTabAt(idx);
        }
    }

    public static boolean isDockTab(final JComponent c)
    {
        if (middleTab == null)
            return false;

        int idx = middleTab.indexOfComponent(c);
        return (idx != -1);
    }

    public static void selectTab(final JComponent c)
    {
        if (middleTab == null)
            return;

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                selectTabSub(c);
            }
        });
    }

    private static void selectTabSub(final JComponent c)
    {
        if (middleTab == null)
            return;

        int idx = middleTab.indexOfComponent(c);
        if (idx != -1)
        {
            middleTab.setSelectedIndex(idx);
            c.requestFocus();
        }
        else
        {
            final JFrame frm = getFrame(c);
            if (frm == null)
                return;
            frm.requestFocus();
        }
    }

    public static void setEditorTitleUpdate(TxtEditorTabPanel c)
    {
        if (middleTab == null)
            return;

        final String t = c.getTabTitle();
        final int idx = middleTab.indexOfComponent(c);
        if (idx != -1)
        {
            if (!middleTab.getTitleAt(idx).equals(t))
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        titleUpdateSub(idx, t);
                    }
                });
            }
        }
        else
        {
            final JFrame frm = getFrame(c);
            if (frm == null)
                return;
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    frm.setTitle(t);
                }
            });
        }
    }

    private static void titleUpdateSub(final int idx , final String t)
    {
        TabOfTab tt = (TabOfTab)middleTab.getTabComponentAt(idx);
        tt.setTitle(t);
    }

    public static TxtEditorTabPanel getSelectedEditor()
    {
        if (middleTab == null)
            return null;

        Component c = middleTab.getSelectedComponent();
        if (c != null
         && c instanceof TxtEditorTabPanel)
            return (TxtEditorTabPanel)c;

        return null;
    }

    //////////////////////////////////////////////////////////////////////////

    static Hashtable<String, ImageIcon> iconMap = new Hashtable<String, ImageIcon>();

    public static ImageIcon getIconImage(String k)
    {
        String nk = "/resource/" + k;
        if (iconMap.containsKey(nk))
            return iconMap.get(nk);

        URL url = Kit.class.getResource(nk);
        if (url ==  null)
            return null;
        ImageIcon icon = new ImageIcon(url);
        iconMap.put(nk, icon);
        return icon;
    }

    //////////////////////////////////////////////////////////////////////////

    public static void setWaitCursor(boolean flag)
    {

        Cursor setting = Cursor.getPredefinedCursor((flag)? Cursor.WAIT_CURSOR
                                                          : Cursor.DEFAULT_CURSOR);

        try
        {
            mainFrame.setCursor(setting);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    //////////////////////////////////////////////////////////////////////////

    static final Object[] optionsYesNo = { "了解",
                                             "取消" };

    public static boolean yesNoDialog(String message, String title)
    {
        int result = JOptionPane.showOptionDialog(mainFrame, message, title,
                                                  JOptionPane.DEFAULT_OPTION,
                                                  JOptionPane.ERROR_MESSAGE,
                                                  null,
                                                  optionsYesNo,
                                                  optionsYesNo[0]);
        return result == 0;
    }

    public static boolean comfirmDialog(String message, String title)
    {
        int result = JOptionPane.showConfirmDialog(mainFrame, message, title,
                                                  JOptionPane.YES_NO_OPTION,
                                                  JOptionPane.INFORMATION_MESSAGE);
        return result == JOptionPane.OK_OPTION;
    }

    public static void messageDialog(String message)
    {
        messageDialog(message, true);
    }

    public static void messageDialog(String message, String title)
    {
         JOptionPane.showOptionDialog(mainFrame, message,
                                      title,
                                      JOptionPane.DEFAULT_OPTION,
                                      JOptionPane.INFORMATION_MESSAGE ,
                                      null,
                                      optionsComfirm,
                                      optionsComfirm[0]);
    }

     static final Object[] optionsComfirm = { "了解" };

    public static void messageDialog(String message, boolean isHtml)
    {

        String html = null;
        if (isHtml == true)
             html = toHTML(message);
        else
             html = message;

         JOptionPane.showOptionDialog(mainFrame, html,
                                      "エラー",
                                      JOptionPane.DEFAULT_OPTION,
                                      JOptionPane.ERROR_MESSAGE,
                                      null,
                                      optionsComfirm,
                                      optionsComfirm[0]);
    }

    private static String toHTML(String msg)
    {
        char [] a = msg.toCharArray();
        StringBuilder rtn = new StringBuilder("<HTML>");
        int letter = 1;

        for (int i = 0; i < a.length; ++i)
        {
             rtn.append(a[i]);

             if (a[i] == '"' )
                 letter *= -1;

             if (a[i] == ':' && letter == 1)
                 rtn.append("<BR>");
        }
        rtn.append("</HTML>");

        return rtn.toString();
    }

    public static File selectFile(String path, int mode)
    {
        System.out.printf("selectFile*dir=%s", path);
        String wpath = path;

        if (wpath == null && mode == 0)
            wpath = selFile;

        JFileChooser fc   = new JFileChooser(wpath);

        int selected;
        if (mode == 0)
            selected = fc.showOpenDialog(mainFrame);
        else
            selected = fc.showSaveDialog(mainFrame);

        if (selected != JFileChooser.APPROVE_OPTION)
            return null;

        File file = fc.getSelectedFile();

        if (mode == 1 && file.exists())
        {
            if (!yesNoDialog("ファイルが存在します. 上書きしますか?" + " ",
                             "確認"))
                return null;
        }

        if (mode == 0)
            selFile = file.getPath();

        return file;
    }

    public static String inputDialog(String message)
    {
        return JOptionPane.showInputDialog(mainFrame, message);
    }

    public static String inputDialog(String message, String initialValue)
    {
        return JOptionPane.showInputDialog(mainFrame,
                                              message,
                                              initialValue);
    }

    public static void setEnabled(Container c, boolean onoff)
    {
        if (c instanceof JComponent)
        {
            ((JComponent)c).setEnabled(onoff);
        }

        Component [] cps = null;
        if (c instanceof JMenu)
            cps = ((JMenu)c).getMenuComponents();
        else
            cps = c.getComponents();

        for (Component cp : cps)
        {
            if (cp instanceof Container)
            {
                Kit.setEnabled((Container)cp, onoff);
            }
        }
    }

    /////////////////////////////////////////////////

    public static void treeNodeExpand(JTree t, DefaultMutableTreeNode node, boolean isExpand)
    {
        doExpandNode(t, node,isExpand);
        switchExpand(t, new TreePath(node.getPath()), isExpand);
    }

    private static void doExpandNode(JTree t, DefaultMutableTreeNode node, boolean isExpand)
    {
        Enumeration e = node.children();
        DefaultMutableTreeNode child = null;
        while(e.hasMoreElements())
        {
            child = (DefaultMutableTreeNode) e.nextElement();
            if (!node.isLeaf())
            {
                doExpandNode(t, child, isExpand);
                switchExpand(t, new TreePath(child.getPath()), isExpand);
            }
        }
    }

    private static void switchExpand(JTree t, TreePath path, boolean isExpand)
    {
        if (isExpand) t.expandPath(path);
        else          t.collapsePath(path);
    }

    /////////////////////////////////////////////////////////////////////

    static public Color cnvColorRGB(String t)
    {
        try
        {
            StringTokenizer tok = new StringTokenizer(t,".");
            int R = Integer.parseInt(tok.nextToken());
            int G = Integer.parseInt(tok.nextToken());
            int B = Integer.parseInt(tok.nextToken());
            return new Color(R,G,B);
        }
        catch(Exception ex)
        {
            return null;
        }
    }

    public static void  closeFrame(Component c)
    {
        final JFrame frm = getFrame(c);
        if (frm == null)
            return;

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                frm.setVisible(false);
                frm.dispose();
            }
        });
    }

    public static JFrame  getFrame(Component c)
    {
        Component cc = c.getParent();
        while(cc != null)
        {
            if (cc instanceof JFrame)
            {
                return (JFrame)cc;
            }
            cc = cc.getParent();
        }
        return null;
    }
}

