//============================================================================
//
//    Java Text Editor library
//
//  MainFrame.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

public class MainFrame extends JFrame
{
    public static MainFrame instance_;

    static String HOME  = "c:/EditorKit";
    JTextField statusField;

    private CustomTabPane  middleTab  = null;

    public MainFrame()
    {
        super();

        EditorConfig.load();

        createInitFrame();

        instance_ = this;

        ArrayList imgs = new ArrayList();
        imgs.add(new ImageIcon(Main.class.getResource("/resource/Icon16x16.gif")).getImage());
        imgs.add(new ImageIcon(Main.class.getResource("/resource/Icon32x32.gif")).getImage());
        this.setIconImages(imgs);
    }

    private void createInitFrame()
    {
        setTitle("JTextEditor");

        JPanel panelRight = new JPanel();
        panelRight.setLayout(new BorderLayout());
        middleTab  = new CustomTabPane();
        middleTab.setMinimumSize(new Dimension(0,0));
        panelRight.add(middleTab, BorderLayout.CENTER);

        Kit.setCostomTab(middleTab);

        statusField = new JTextField();
        statusField.setEditable(false);

        JMenuBar menubar = createMenuBar();
        setJMenuBar(menubar);

        JToolBar toolbar = createToolBar();
        JPanel toolbarBanel = new JPanel();
        toolbarBanel.setLayout(new BorderLayout());
        toolbarBanel.add(toolbar, BorderLayout.CENTER);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(toolbarBanel,    BorderLayout.NORTH);
        getContentPane().add(panelRight,      BorderLayout.CENTER);
        getContentPane().add(statusField,     BorderLayout.SOUTH);
        getContentPane().setPreferredSize(new Dimension(850,   850));

        pack();

        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(size.width /2 - getSize().width /2,
                    size.height/2 - getSize().height/2);

        setListener();
    }


    private void setListener()
    {
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE );

        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e){ close(); }
        });

        ActionListener newAct =new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                newFile();
            }
        };
        newItem.addActionListener(newAct);
        newBtn.addActionListener(newAct);

        ActionListener openAct =new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                openFile();
            }
        };
        openItem.addActionListener(openAct);
        openBtn.addActionListener(openAct);

        ActionListener saveAct =new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                saveCurrent();
            }
        };
        saveItem.addActionListener(saveAct);
        saveBtn.addActionListener(saveAct);

        saveAsItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                saveAsFile();
            }
        });

        ActionListener saveAllAct =new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                saveAll();
            }
        };
        saveAllItem.addActionListener(saveAllAct);
        saveAllBtn.addActionListener(saveAllAct);

        closeAllItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                closeTabAll();
            }
        });

        exitItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                close();
            }
        });

        ActionListener grepAct =new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                doGrep();
            }
        };
        grepItem.addActionListener(grepAct);
        grepBtn.addActionListener(grepAct);

        ActionListener findAct =new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                doFind();
            }
        };
        findItem.addActionListener(findAct);
        findBtn.addActionListener(findAct);

        DropTargetListener dtl = new DropTargetAdapter() 
        {
           public void dragOver(DropTargetDragEvent dtde)
           {
                if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
                {
                    dtde.acceptDrag(DnDConstants.ACTION_COPY);
                    return;
                }
                dtde.rejectDrag();
            }

            public void drop(DropTargetDropEvent dtde)
            {
                try
                {
                    if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
                    {
                        dtde.acceptDrop(DnDConstants.ACTION_COPY);
                        Transferable transferable = dtde.getTransferable();
                        java.util.List list = (java.util.List)(transferable.getTransferData(DataFlavor.javaFileListFlavor));
                        for (int i=0;i<list.size();++i)
                             fileDrop((File)list.get(i));

                        dtde.dropComplete(true);
                        return;
                    }
                }
                catch(UnsupportedFlavorException ufe)
                {
                    ufe.printStackTrace();
                }
                catch(IOException ioe)
                {
                    ioe.printStackTrace();
                }
                dtde.rejectDrop();
            }
        };
        new DropTarget(this, DnDConstants.ACTION_COPY, dtl, true);
    }

    private void fileDrop(File f)
    {
        if (f.isDirectory())
        {
            for(File ff : f.listFiles())
            {
                 fileDrop(ff);
            }
        }
        else
        {
            openEditFile(f);
        }
    }

    public void close()
    {
        setVisible(false);
        this.dispose();
        System.exit(0);
    }

    /////////////////////////////////////

    JMenuItem newItem;
    JMenuItem openItem;
    JMenuItem saveItem;
    JMenuItem saveAsItem;
    JMenuItem saveAllItem;
    JMenuItem closeAllItem;

    JMenuItem exitItem;

    JMenuItem grepItem;
    JMenuItem findItem;

    protected JMenuBar createMenuBar()
    {
        JMenuBar menuBar  = new JMenuBar();

        //-----------------------
        JMenu menuFile      = new JMenu("ファイル(F)");

        JMenu menuEdit      = new JMenu("編集(E)");

        newItem      = new JMenuItem("新規(N)",             Kit.getIconImage("NewFile.GIF"));
        openItem     = new JMenuItem("開く(O)",             Kit.getIconImage("Open.GIF"));
        saveItem     = new JMenuItem("上書き保存(S)",       Kit.getIconImage("SaveFile.GIF"));
        saveAsItem   = new JMenuItem("名前を付けて保存(A)", Kit.getIconImage("SaveAs.GIF"));
        saveAllItem  = new JMenuItem("全てを保存(Z)",       Kit.getIconImage("SaveAll.GIF"));
        closeAllItem  = new JMenuItem("全て閉じる(C)");
        exitItem     = new JMenuItem("終了(X)");

        grepItem     = new JMenuItem("Grep(G)", Kit.getIconImage("Search.GIF"));
        findItem     = new JMenuItem("Find(F)", Kit.getIconImage("Find.GIF"));

        menuFile.setMnemonic(KeyEvent.VK_F);
        menuEdit.setMnemonic(KeyEvent.VK_E);
        newItem.setMnemonic(KeyEvent.VK_N);
        newItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
        openItem.setMnemonic(KeyEvent.VK_O);
        openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
        saveItem.setMnemonic(KeyEvent.VK_S);
        saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
        saveAsItem.setMnemonic(KeyEvent.VK_A);
        saveAsItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK|InputEvent.SHIFT_MASK));
        saveAllItem.setMnemonic(KeyEvent.VK_Z);

        closeAllItem.setMnemonic(KeyEvent.VK_C);
        closeAllItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK|InputEvent.SHIFT_MASK));

        exitItem.setMnemonic(KeyEvent.VK_X);

        grepItem.setMnemonic(KeyEvent.VK_E);
        grepItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, InputEvent.CTRL_MASK));

        findItem.setMnemonic(KeyEvent.VK_F);
        findItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK));

        menuFile.add(newItem);
        menuFile.add(openItem);
        menuFile.add(new JSeparator());
        menuFile.add(saveItem);
        menuFile.add(saveAsItem);
        menuFile.add(saveAllItem);
        menuFile.add(new JSeparator());
        menuFile.add(closeAllItem);
        menuFile.add(new JSeparator());
        menuFile.add(exitItem);

        menuEdit.add(grepItem);
        menuEdit.add(findItem);

        menuBar.add(menuFile);
        menuBar.add(menuEdit);
        return menuBar;
    }

    /////////////////////////////////////


    JButton newBtn;
    JButton openBtn;
    JButton saveBtn;
    JButton saveAllBtn;

    JButton grepBtn;
    JButton findBtn;

    private JToolBar createToolBar()
    {
        JToolBar toolBar = new JToolBar("ツール");
        newBtn = new JButton (Kit.getIconImage("NewFile.GIF"));
        newBtn.setToolTipText("新規");
        newBtn.setOpaque(false);
        openBtn = new JButton (Kit.getIconImage("Open.GIF"));
        openBtn.setToolTipText("開く");
        openBtn.setOpaque(false);
        saveBtn = new JButton (Kit.getIconImage("SaveFile.GIF"));
        saveBtn.setToolTipText("上書き保存");
        saveBtn.setOpaque(false);
        saveAllBtn = new JButton (Kit.getIconImage("SaveAll.GIF"));
        saveAllBtn.setToolTipText("全てを保存");
        saveAllBtn.setOpaque(false);

        grepBtn = new JButton (Kit.getIconImage("Search.GIF"));
        grepBtn.setToolTipText("Grep");
        grepBtn.setOpaque(false);
        findBtn = new JButton (Kit.getIconImage("Find.GIF"));
        findBtn.setToolTipText("Find");
        findBtn.setOpaque(false);

        toolBar.add(newBtn);
        toolBar.add(openBtn);
        toolBar.add(saveBtn);
        toolBar.add(saveAllBtn);

        toolBar.add(grepBtn);
        toolBar.add(findBtn);

        return toolBar;
    }

    /////////////////////////////////////

    public void resetStatusText()
    {
        setStatusText("");
    }

    public void setStatusText(final String s)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                statusField.setText(s);
            }
        });
    }

    /////////////////////////////////////////////////////

    TxtEditorTabPanel newFile()
    {
        String preFix = "新規ファイル";
        int i = 1;
        String nm;
        while(true)
        {
            nm = String.format("%s%d", preFix, i++);
            if (!KitManager.containsNewFile(nm))
                break;
        }
        return openEditor(new File(nm), ".txt", 1);
    }

    void openFile()
    {
        JFileChooser fileChooser = new JFileChooser(); 
        fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        fileChooser.setMultiSelectionEnabled(true);
        if (fileChooser.showOpenDialog(MainFrame.instance_) != JFileChooser.APPROVE_OPTION)
            return;

        File [] src = fileChooser.getSelectedFiles();
        if (src != null)
        {
            for(File ff : src)
                fileDrop(ff);
        }
    }

    public static void openEditFile(File f)
    {
        int idx = f.getName().lastIndexOf(".");
        String ext = ".txt";
        if (idx > 0)
            ext = f.getName().substring(idx);

        TxtEditorTabPanel ed = KitManager.getTextEditor(f.getPath().replace("\\", "/"));
        if (ed == null)
        {
            if (HighLightControler.containsExtEntry(ext)) 
            {
                openEditor(f, ext, 2);
            }
            else
            {
                openEditor(f, ".txt", 2);
            }
        }
        else
        {
            final TxtEditorTabPanel ee = ed; 
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    Kit.selectTab(ee);
                }
            });
        }
    }

    public static TxtEditorTabPanel openEditor(final File f, final String ext, int mode)
    {
        final TxtEditorTabPanel edit = new TxtEditorTabPanel(ext);
        Kit.addTab(f.getName(), edit);
        KitManager.addEditor(edit);
        if (mode == 1)
        {
            edit.setFile(f);
            edit.mode = 1;
            return edit;
        }

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                edit.openFile(f);
            }
        });

        return edit;
    }

    void saveCurrent()
    {
        TxtEditorTabPanel ed = Kit.getSelectedEditor();
        if (ed == null)
            return;

        if (ed.isEdited)
        {
            if (ed.writeCheckTimeStamp())
                ed.saveSilent();
        }
    }

    void saveAsFile()
    {
        TxtEditorTabPanel ed = Kit.getSelectedEditor();
        if (ed == null)
            return;

        if (ed.newName())
        {
            if (ed.writeCheckTimeStamp())
                ed.saveSilent();
        }
    }

    void saveAll()
    {
        KitManager.saveAll();
    }

    void closeTabAll()
    {
        KitManager.closeAll();
    }

    //////////////////////////////////////////////////////////////////

    static class TxtResultAct implements IResult
    {
        TxtEditorTabPanel ed;
        public TxtResultAct(TxtEditorTabPanel ed)
        {
            this.ed = ed;
        }

        public  File getSrcFolder()
        {
            return new File(HOME);
        }

        public void addResult(SearchType type, String title, ArrayList<String[]> args)
        {
            StringBuilder sb = new StringBuilder();
            sb.append("\n");
            if (type == SearchType.SOURCE)
            {
                sb.append(title);
                sb.append("\n");
                for(String [] ll : args)
                {
                    sb.append(String.format("(%s:%s) %s", ll[0],ll[1],ll[2]));
                    sb.append("\n");
                }
            }
            else
            {
                sb.append(title);
                sb.append("\n");
                for(String [] ll : args)
                {
                    sb.append(ll[0]);
                    sb.append("\n");
                }
            }
            ed.editorPanel.tools.insertString(sb.toString());
        }
    }

    private void doGrep()
    {
        TxtEditorTabPanel ed = newFile();
        GrepDialog dlg = new GrepDialog(new TxtResultAct(ed));
        dlg.setVisible(true);
    }

    private void doFind()
    {
        TxtEditorTabPanel ed = newFile();
        FindDialog dlg = new FindDialog(new TxtResultAct(ed));
        dlg.setVisible(true);
    }
}
