//============================================================================
//
//    Java Text Editor library
//
//    TxtEditorPanel.java
//
//  Copyright (c) 2011 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import javax.swing.plaf.*;
import javax.swing.text.*;
import javax.swing.undo.*;

import static text.editor.Kit.*;

public class TxtEditorPanel  extends JPanel 
{
    public SourceToolBar         tools;
    public JEditorPane           edit;
    public LNView                 lnview;
    public DefaultStyledDocument doc;

    public JPopupMenu pop        = null;
    public JMenuItem  cutItem    = null;
    public JMenuItem  copyItem   = null;
    public JMenuItem  pasteItem  = null;
    public JMenuItem  searchItem = null;

    public TxtEditorPanel(String ext)
    {
        edit  = new CustomEditorPane();
        edit.setEditorKit(new CustomEditorKit());
        edit.setBackground(EditorConfig.bgColor);
        edit.setForeground(EditorConfig.fgColor);
        edit.setFont(EditorConfig.font);
        edit.setEditable(true);

        tools = new SourceToolBar((JTextPane)edit, ext);
        doc = (DefaultStyledDocument)edit.getDocument();
        doc.putProperty("tabSize", Integer.valueOf(EditorConfig.tabSize));

        JPanel edtPanel = new JPanel();
        JScrollPane scroll = new JScrollPane(edit);
        lnview = new LNView(edit);
        scroll.setRowHeaderView(lnview);
        tools.setLineNumberView(lnview);

        setLayout(new BorderLayout());
        add(tools,   BorderLayout.NORTH);
        add(scroll, BorderLayout.CENTER);

        createPopup();
    }

    private void createPopup()
    {
        pop = new JPopupMenu();

        searchItem    = new JMenuItem("検索(S)",     getIconImage("Search.GIF"));
        copyItem      = new JMenuItem("コピー(C)",   getIconImage("Copy.GIF"));
        cutItem       = new JMenuItem("切り取り(T)", getIconImage("Cut.GIF"));
        pasteItem     = new JMenuItem("貼り付け(P)", getIconImage("Paste.GIF"));

        searchItem.setMnemonic(KeyEvent.VK_S);
        copyItem.setMnemonic(KeyEvent.VK_T);
        cutItem.setMnemonic(KeyEvent.VK_C);
        pasteItem.setMnemonic(KeyEvent.VK_P);

        pop.add(searchItem);
        pop.add(copyItem);
        pop.add(cutItem);
        pop.add(pasteItem);

        searchItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                tools.openSearchDialog();
            }
        });

        cutItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                edit.cut();
            }
        });

        copyItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                edit.copy();
            }
        });

        pasteItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                edit.paste();
            }
        });

        edit.addMouseListener(new MouseAdapter()
        {
            public void mouseClicked(MouseEvent e)
            {
                if (e.getClickCount() == 1)
                {
                    if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0)
                        pop.show(e.getComponent(),e.getX(),e.getY());
                }
            }
        });

    }

    ///////////////////////////////////////////////////////////////////////////////////

    static public String read(File f) 
    {
        String enc ;
        if (EditorConfig.inEncode.equals("SJIS(CRLF)"))
            enc = "Shift_JIS";
        else if (EditorConfig.inEncode.equals("EUC(LF)"))
            enc = "EUC_JP";
        else
            enc = "UTF-8";

        StringBuffer sb = new StringBuffer();
        InputStreamReader rd = null;
        try
        {
            rd = new InputStreamReader(
                       new FileInputStream(f.getAbsolutePath()), enc);
            int ch;
            while ((ch = rd.read()) != -1) 
                sb.append((char)ch);
        }
        catch(Exception ex)
        {
        }
        finally
        {
            try
            {
                if (rd != null)
                    rd.close();
            }
            catch(Exception ex){}
        }
        return toLF(sb);
    }

    public static void write(File f, String text)
    {
         String enc ;
        if (EditorConfig.outEncode.equals("SJIS(CRLF)"))
            enc = "Shift_JIS";
        else if (EditorConfig.outEncode.equals("EUC(LF)"))
            enc = "EUC_JP";
        else
            enc = "UTF-8";

        StringBuffer sb = null;
        if (EditorConfig.outTab2space)
        {
            sb = tab2Spc(text, EditorConfig.tabSize);
        }
        else
        {
            sb = new StringBuffer(text);
        }

        BufferedWriter wr = null;
        try
        {
            wr = new BufferedWriter(new OutputStreamWriter(
                                     new FileOutputStream(f.getAbsolutePath()), enc));

            if (enc.equals("Shift_JIS"))
                wr.write(toCRLF(sb));
            else
                wr.write(toLF(sb));
            wr.close();
        }
        catch(Exception ex)
        {
        }
        finally
        {
            try
            {
                if (wr != null)
                    wr.close();
            }
            catch(Exception ex){}
        }
    }

    private static String toCRLF(StringBuffer sb )
    {
        return sb.toString().replace("\n", "\r\n");
    }

    private static String toLF(StringBuffer sb)
    {
        return sb.toString().replace("\r", "");
    }

    public static StringBuffer tab2Spc(String s, int tabSize)
    {
        StringBuffer sb = new StringBuffer();
        
        int len = s.length();
        int pos = 0;
        for (int i = 0; i < len; ++i)
        {
            char cc = s.charAt(i);
            if (cc == '\t')
            {
                int n = tabSize - (pos % tabSize);
                for(int j = 0; j < n; ++j)
                    sb.append(' ');
                pos += n;
            }
            else if (cc == '\n')
            {
                sb.append(cc);
                pos = 0;
            }
            else
            {
                sb.append(cc);
                pos++;
            }
        }
        return sb;
    }
    /////////////////////////////////////////////////////////////////////////////////

    public void jumpCommand(String m)
    {
        try
        {
            String [] col = m.split(":");
            if (col.length != 2)
            {
                return;
            }
            tools.setCaretPosition(0);
            try
            {
                int r = Integer.parseInt(col[0]);
                int c = Integer.parseInt(col[1]);
                int x = tools.getLineStartPos(r);
                x += c;
                tools.setCaretPosition(x);
            }
            catch(Exception ex)
            {
            }
            edit.requestFocus();
            edit.validate();
        }
        catch(Exception ex)
        {
        }
    }

}

