//============================================================================
//
//    Java Text Editor library
//
//    TxtEditorTabPanel.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import javax.swing.plaf.*;
import javax.swing.text.*;
import javax.swing.undo.*;

public class TxtEditorTabPanel  extends JPanel 
                               implements ISyncCloseable
{
    public TxtEditorPanel editorPanel  = null;
    public JEditorPane edit  = null;

    long    openMtime;
    File    file = null;

    public String getFilePath()
    {
        if (file == null)
            return "";
        return file.getAbsolutePath().replace('\\', '/');
    }

    public void setFile(File ofile)
    {
        file = ofile;
        if (file == null)
            return;
    }

    public void changeFile(File f)
    {
        setFile(f);
        if (file == null)
            return;

        openMtime = file.lastModified(); 
        Kit.setEditorTitleUpdate(this);
    }

    int mode = 0; /* 0: init 1:new 2:open */

    public String getPanelTitle()
    {
        if (file == null)
           return "";
        else
            return file.getName();
    }

    public String getTabTitle()
    {
        String t = getPanelTitle();
        if (isEdited)
            t += "*";
        return t;
    }

    public TxtEditorTabPanel(String ext)
    {
        editorPanel  = new TxtEditorPanel(ext);
        edit  = editorPanel.edit;
        setLayout(new BorderLayout());
        add(editorPanel,           BorderLayout.CENTER);

        newFile();

        docListener = new CustomDocListener();
        edit.getDocument().addDocumentListener(docListener);
    }

    public void jumpCommand(String m)
    {
        editorPanel.jumpCommand(m);
    }

    public void refresh()
    {
        editorPanel.tools.resetProperties();
        invalidate();
        repaint();
    }

    class CustomDocListener implements DocumentListener
    {
        boolean pending = false;
        long    t;

        public void setPending(boolean onoff)
        {
            pending = onoff;
            t = System.currentTimeMillis();
        }

        private void checkTimeout()
        {
            if (!pending)
                return;

            long n = System.currentTimeMillis();
            if (n - t > 15 * 1000)
                pending = false;
        }

        public void changedUpdate(DocumentEvent de)
        {
            checkTimeout();
            if (!pending)
                setEdited(true);
        }

        public void insertUpdate(DocumentEvent de)
        {
            checkTimeout();
            if (!pending)
                setEdited(true);
        }

        public void removeUpdate(DocumentEvent de)
        {
            checkTimeout();
            if (!pending)
                setEdited(true);
        }
    };

    private ActionListener templateAct;
    private CustomDocListener docListener;
    private ActionListener loadEndAct;
 
    private void setMenuAction()
    {
        edit.addMouseListener(new  MouseAdapter()
        {
            public void mousePressed(MouseEvent e)
            {
                if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0)
                {
                }
            }

            public void mouseClicked(MouseEvent e)
            {
               if (e.getClickCount() == 2)
               {
               }
            }
        });

        FocusListener focus = new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
            }
            public void focusLost(FocusEvent e)
            {
            }
        };

        edit.addFocusListener(focus);
    }

    public boolean isEdited = false;

    public void setEdited(boolean onoff)
    {
        isEdited = onoff;
        Kit.setEditorTitleUpdate(this);
    }

    public void reload()
    {
        if (!canDestroy())
            return;

        openFile(file);
    }

    public boolean canDestroy()
    {
        if (isEdited)
        {
            boolean result = Kit.yesNoDialog(
                                     "編集中のデータは破棄されます.よろしいですか?",
                                     "確認");
            if (result == false)
               return false;
        }

        mode = 0;
        return true;
    }

    public void newFile()
    {
        if (!canDestroy())
            return;

        file = null;
        edit.setText("");

        mode = 1;
    }

    public void openFile(File ofile)
    {
        setFile(ofile);
        if (file == null)
            return;

        try
        {
            read();
        }
        catch(Exception ex){ ex.printStackTrace(); }
        mode = 2;
        openMtime = file.lastModified(); 
    }

    public boolean writeCheckTimeStamp()
    {
        File ff = new File(file.getPath());
        if (!ff.exists())
        {
            if (mode == 1)
                return newName();
            else
                return Kit.comfirmDialog(file.getName() 
                                          + "\nは存在しません、削除されている可能性があります."
                                          + "\n保存しますか？",
                                            "確認");
        }

        long stmp = ff.lastModified();
        if (stmp > openMtime)
        {
            return Kit.comfirmDialog(file.getName() 
                                      + "\nは更新されている可能性があります."
                                      + "\nこのまま保存すると更新が無効になり"
                                      + "\nエラーの原因になります."
                                      + "\n後から保存し直すこともできるので\n"
                                      + "<いいえ>をお勧めします.\n"
                                      + "\n保存しますか？",
                                        "確認");
        }
        return true;
    }

    public boolean newName()
    {
        JFileChooser fileChooser = new JFileChooser(); 
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setMultiSelectionEnabled(false);
        if (fileChooser.showSaveDialog(MainFrame.instance_) != JFileChooser.APPROVE_OPTION)
            return false;

        File dst = fileChooser.getSelectedFile();
        if (dst.exists())
        {
            if (!Kit.comfirmDialog("同名のファイルがあります.\n上書きしますか？","確認"))
                return false;
        }
        changeFile(dst);
        return true;
    }

    public void saveSilent()
    {
        if (file == null)
            return;

        write(file);
        mode = 0;
        openMtime =  new File(file.getPath()).lastModified(); 
    }

    ///////////////////////////////////////////////////////////////////////////////////

    public void read()
    {
        try
        {
            editorPanel.tools.initText(TxtEditorPanel.read(file));
            edit.setCaretPosition(0);
            editorPanel.tools.undoClear();
            setEdited(false);
        }
        catch(Exception ex){ ex.printStackTrace(); }
    }

    public void write(File f)
    {
        TxtEditorPanel.write(f, edit.getText());
        setEdited(false);
    }

    /////////////////////////////////////////////////////////////////////////////////

    public boolean disposed = false;

    public boolean tabClose()
    {
        if (!canDestroy())
            return false;

        if (!disposed)
        {
            KitManager.removeEditor(this);
            editorPanel.tools.dispose();
            Document doc = edit.getDocument();
            if (doc instanceof HighLightedDocument)
            {
                ((HighLightedDocument)doc).dispose();
            }
        }
        disposed = true;

        return true;
    }
}

