//============================================================================
//
//    Java Text Editor library
//
//    EditSettingDialog.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.text.*;

import static text.editor.Kit.*;

public class EditSettingDialog extends JFrame
{
    Font setfont = null;

    private SourceToolBar toolbar = null;

    private JTextField  fontText= null;
    private JButton     fontRef = null;

    private JComboBox   tabsizeCombo = null;

    private JTextField  textColor = null;
    private JButton     colorRef1 = null;

    private JTextField  bgColor = null;
    private JButton     colorRef2 = null;

    private JTextField  graphColor = null;
    private JButton     colorRef3 = null;

    private JCheckBox   popAssistChk = null;

    private JComboBox   inputCombo = null;
    private JComboBox   outputCombo = null;

    private JCheckBox   convTabChk = null;
    private JCheckBox   keyTabChk = null;

    private CustomTabPane  tabs  = null;

    private JButton okBtn;
    private JButton cancelBtn;

    public EditSettingDialog(SourceToolBar toolbar)
    {
        super("テキストエデイタの設定");

        setIconImage(Kit.getIconImage("Setting.GIF").getImage());

        this.toolbar = toolbar;

        tabs = new CustomTabPane();
        tabs.addTab("表示",   null, new JScrollPane(createPanel()));
        tabs.addTab("入出力", null, new JScrollPane(createIOPanel()));

        getContentPane().add( tabs, BorderLayout.CENTER);

        JPanel bottm = new JPanel();
        bottm.setBorder(new EmptyBorder(4,4,4,4));
        bottm.setLayout(new BoxLayout(bottm, BoxLayout.X_AXIS));
        bottm.add(Box.createHorizontalGlue());
        okBtn = new JButton("適用");
        bottm.add(okBtn);
        bottm.add(Box.createRigidArea(new Dimension(8,8)));
        cancelBtn = new JButton("キャンセル");
        bottm.add(cancelBtn);
        bottm.add(Box.createHorizontalGlue());
        getContentPane().add( bottm, BorderLayout.SOUTH);

        setListener();

        setSize(500, 500);

        setFieldsInit();

        setAlwaysOnTop(true);
    }

    public void setSourceToolBar(SourceToolBar toolbar)
    {
        this.toolbar = toolbar;
    }

    String[] tabsizeData = {"2",    "4", "8"};
    String[] encodeData  = {"UTF-8(LF)", "SJIS(CRLF)", "EUC(LF)" };

    protected JComponent createPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(null);

        int x = 20;
        int y = 20;
        JLabel l1 = new JLabel("フォント");
        l1.setBounds(x, y , 70, 22);
        panel.add(l1);
        fontText    = new JTextField();
        fontText.setBounds(x+70, y , 250, 22);
        panel.add(fontText);
        fontRef = new JButton("...");
        fontRef.setBounds(x+320, y , 30, 22);
        panel.add(fontRef);

        y += 35;
        JLabel l2 = new JLabel("タブサイズ");
        l2.setBounds(x, y , 70, 22);
        panel.add(l2);
        tabsizeCombo = new JComboBox(new DefaultComboBoxModel(tabsizeData));
        tabsizeCombo.setBounds(x+70, y , 250, 22);
        panel.add(tabsizeCombo);

        y += 35;
        JLabel l3 = new JLabel("テキスト色");
        l3.setBounds(x, y , 70, 22);
        panel.add(l3);
        textColor    = new JTextField();
        textColor.setBounds(x+70, y , 250, 22);
        panel.add(textColor);
        colorRef1 = new JButton("...");
        colorRef1.setBounds(x+320, y , 30, 22);
        panel.add(colorRef1);

        y += 35;
        JLabel l4 = new JLabel("背景色");
        l4.setBounds(x, y , 70, 22);
        panel.add(l4);
        bgColor    = new JTextField();
        bgColor.setBounds(x+70, y , 250, 22);
        panel.add(bgColor);
        colorRef2 = new JButton("...");
        colorRef2.setBounds(x+320, y , 30, 22);
        panel.add(colorRef2);

        y += 35;
        JLabel l5 = new JLabel("記号色");
        l5.setBounds(x, y , 70, 22);
        panel.add(l5);
        graphColor    = new JTextField();
        graphColor.setBounds(x+70, y , 250, 22);
        panel.add(graphColor);
        colorRef3 = new JButton("...");
        colorRef3.setBounds(x+320, y , 30, 22);
        panel.add(colorRef3);

        return panel;
    }

    protected JComponent createIOPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(null);
        int x = 20;
        int y = 20;

        JLabel l6 = new JLabel("入力時のエンコーディング");
        l6.setBounds(x, y , 250, 22);
        panel.add(l6);
        y += 35;
        inputCombo = new JComboBox(new DefaultComboBoxModel(encodeData));
        inputCombo.setBounds(x, y , 250, 22);
        panel.add(inputCombo);

        y += 35;
        keyTabChk = new JCheckBox("Key入力でのタブを空白に変換.");
        keyTabChk.setBounds(x, y , 250, 22);
        panel.add(keyTabChk);

        y += 35 * 2;
        JLabel l7 = new JLabel("出力時のエンコーディング");
        l7.setBounds(x, y , 250, 22);
        panel.add(l7);

        y += 35;
        outputCombo = new JComboBox(new DefaultComboBoxModel(encodeData));
        outputCombo.setBounds(x, y , 250, 22);
        panel.add(outputCombo);

        y += 35;
        convTabChk = new JCheckBox("ファイル出力時にタブを空白に変換.");
        convTabChk.setBounds(x, y , 250, 22);
        panel.add(convTabChk);

        return panel;
    }

    private void setFieldsInit()
    {
        fontText.setText(EditorConfig.fontStr);
        tabsizeCombo.setSelectedItem(EditorConfig.tabSizeStr);
        textColor.setText(EditorConfig.fgColorStr);
        bgColor.setText(EditorConfig.bgColorStr);
        graphColor.setText(EditorConfig.graphColorStr);

        inputCombo.setSelectedItem(EditorConfig.inEncode);
        keyTabChk.setSelected(EditorConfig.keyTab2space);
        outputCombo.setSelectedItem(EditorConfig.outEncode);
        convTabChk.setSelected(EditorConfig.outTab2space);
    }

    protected void setListener()
    {
        fontRef.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               showFontDialog();
            }
        });

        colorRef1.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               showColorDialog(textColor);
            }
        });

        colorRef2.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               showColorDialog(bgColor);
            }
        });

        colorRef3.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               showColorDialog(graphColor);
            }
        });

        okBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               settingOK();
            }
        });

        cancelBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               setVisible(false);
            }
        });
    }

    private void showFontDialog()
    {
        FontDialog f = new FontDialog(this, EditorConfig.font);
        f.setVisible(true);
        Font font = f.getResultFont();
        if (font == null)
            return;
        setfont = font;
        fontText.setText(f.getResult());
    }

    private void showColorDialog(JTextField tgt)
    {
        Color c = JColorChooser.showDialog(this,"color", null);
        if (c == null)
           return;

        String cRGB = "" + c.getRed() + "." + c.getGreen() + "." + c.getBlue();
        tgt.setText(cRGB);
    }

    void settingOK()
    {
        if (!fontText.getText().isEmpty())
            EditorConfig.fontStr = fontText.getText().replace(':','-');

        Object sel= tabsizeCombo.getSelectedItem();
        if (sel != null)
        {
            EditorConfig.tabSizeStr = (String)sel;
        }

        if (!textColor.getText().isEmpty())
            EditorConfig.fgColorStr = textColor.getText();

        if (!bgColor.getText().isEmpty())
            EditorConfig.bgColorStr = bgColor.getText();

        if (!graphColor.getText().isEmpty())
            EditorConfig.graphColorStr = graphColor.getText();

        sel= inputCombo.getSelectedItem();
        if (sel != null)
        {
            EditorConfig.inEncode = (String)sel;
        }
        sel= outputCombo.getSelectedItem();
        if (sel != null)
        {
            EditorConfig.outEncode = (String)sel;
        }

        EditorConfig.keyTab2space = keyTabChk.isSelected();
        EditorConfig.outTab2space = convTabChk.isSelected();

        EditorConfig.cnvTypeValue();
        EditorConfig.save();

        KitManager.refresh();

        setVisible(false);
    }
}

