//============================================================================
//
//    Java Text Editor library
//
//    EditorConfig.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.io.*;
import java.util.*;

public class EditorConfig
{
    private static String HOME;
    private static String FILE;

    private static final String FONT_TAG       = "font";
    private static final String TAB_SIZE       = "tabsize";
    private static final String FG_COLOR       = "fg.color";
    private static final String BG_COLOR       = "bg.color";
    private static final String GRAPH_COLOR    = "graph.color";
    private static final String INPUT_ENCODE   = "in.encode";
    private static final String OUTPUT_ENCODE  = "out.encode";
    private static final String KEY_TAB2SPC    = "key.tab2space";
    private static final String OUT_TAB2SPC    = "out.tab2space";

    static String   fontStr        = "ＭＳ ゴシック-PLAIN-12";
    static String   tabSizeStr     = "4";
    static String   fgColorStr     = "0.0.0";
    static String   bgColorStr     = "255.255.255";
    static String   graphColorStr  = "130.140.120";
    static String   inEncode       = "UTF-8(LF)";
    static String   outEncode      = "UTF-8(LF)";
    static boolean  keyTab2space   = false;
    static boolean  outTab2space   = false;

    public static Font  font       = new Font("ＭＳ ゴシック", Font.PLAIN, 12);
    public static int   tabSize      = 4;
    public static Color bgColor    = Color.WHITE;
    public static Color fgColor    = Color.BLACK;
    public static Color graphColor = Color.LIGHT_GRAY;

    static public void  initialize()
    {
        HOME = Kit.HOME;
        FILE = HOME + "/config/settings/editor.ini";
    }

    static public void  cnvTypeValue()
    {
        try
        {
            font    = Font.decode(fontStr);
            tabSize = Integer.parseInt(tabSizeStr);
            fgColor = cnvColorRGB(fgColorStr);
            bgColor = cnvColorRGB(bgColorStr);
            graphColor = cnvColorRGB(graphColorStr);
        }
        catch(Exception e){}
    }

    static public Color cnvColorRGB(String t)
    {
        try
        {
            StringTokenizer tok = new StringTokenizer(t,".");
            int R = Integer.parseInt(tok.nextToken());
            int G = Integer.parseInt(tok.nextToken());
            int B = Integer.parseInt(tok.nextToken());
            return new Color(R,G,B);
        }
        catch(Exception ex)
        {
            return null;
        }
    }

    static public void load()
    {
        Properties prop = new Properties();
        InputStreamReader in = null;

        try
        { 
            create();
            in = new InputStreamReader(new FileInputStream(FILE), "UTF-8");
            prop.load(in);

            String v  = prop.getProperty(FONT_TAG);
            if (v != null)
                 fontStr = v;

            v  = prop.getProperty(TAB_SIZE);
            if (v != null)
                tabSizeStr = v;

            v  = prop.getProperty(FG_COLOR);
            if (v != null)
                fgColorStr = v;

            v = prop.getProperty(BG_COLOR);
            if (v != null)
                bgColorStr = v; 

            v = prop.getProperty(GRAPH_COLOR);
            if (v != null)
                graphColorStr = v;

            v = prop.getProperty(INPUT_ENCODE);
            if (v != null)
                inEncode = v;

            v = prop.getProperty(OUTPUT_ENCODE);
            if (v != null)
                outEncode = v;

            v = prop.getProperty(KEY_TAB2SPC);
            if (v != null && v.compareToIgnoreCase("true") == 0) 
                keyTab2space = true;

            v = prop.getProperty(OUT_TAB2SPC);
            if (v != null && v.compareToIgnoreCase("true") == 0) 
                outTab2space = true;
        }
        catch(Exception ex)
        {
        } 
        finally
        {
            try
            {
                if (in != null)
                    in.close();
            }
            catch(Exception ex){}
        }
        cnvTypeValue();
    }

    static public void create()
    {
        File f = new File(FILE);
        if (!f.exists())
            save();
    }

    static public void save()
    {
        File f = new File(FILE);
        OutputStream os = null;
        Writer w = null;
        BufferedWriter bw = null;
        try
        {
            os = new FileOutputStream(f);
            w  = new OutputStreamWriter(os, "UTF-8");
            bw = new BufferedWriter(w);

            bw.write(FONT_TAG      + ":" + fontStr);
            bw.newLine();
            bw.write(TAB_SIZE      + ":" + tabSizeStr);
            bw.newLine();
            bw.write(FG_COLOR    + ":" + fgColorStr);
            bw.newLine();
            bw.write(BG_COLOR      + ":" + bgColorStr);
            bw.newLine();
            bw.write(GRAPH_COLOR   + ":" + graphColorStr);
            bw.newLine();
            bw.write(INPUT_ENCODE  + ":" + inEncode);
            bw.newLine();
            bw.write(OUTPUT_ENCODE + ":" + outEncode);
            bw.newLine();
            bw.write(KEY_TAB2SPC   + ":" + keyTab2space);
            bw.newLine();
            bw.write(OUT_TAB2SPC   + ":" + outTab2space);
            bw.newLine();
        }
        catch(Exception ex)
        {
        }
        finally
        {
            if (bw != null) try { bw.close(); } catch (IOException e) {}
            if (w  != null) try { w .close(); } catch (IOException e) {}
            if (os != null) try { os.close(); } catch (IOException e) {}
        }
    }
}
