//============================================================================
//
//    Java Text Editor library
//
//    FindDialog.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;

import java.util.regex.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import text.editor.*;

import static text.editor.Kit.*;

public class FindDialog extends JFrame
{
    private JTextField    filePtn= null;
    private JTextField    folder = null;

    private JCheckBox     wordChk  = null;
    private JCheckBox     caseChk  = null;
    private JCheckBox     regChk   = null;
    private JComboBox     maxCombo = null;

    private JButton       refBtn = null;

    private JButton       searchBtn = null;
    private JButton       cancelBtn = null;

    private JLabel        statusLabel = null;

    private final Object [] comboItems = new Object [] { "100", "250", "500", "1000"};

    private IResult resultAct = null;

    public FindDialog(IResult resultAct)
    {
        super("Find");

        this.resultAct = resultAct;

        setIconImage(Kit.getIconImage("Find.GIF").getImage());

        createField();

        setListener();

        setSize(448, 210);

        setResizable(false);

        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(size.width /2 - getSize().width /2,
                    size.height/2 - getSize().height/2);

        setAlwaysOnTop(true);
    }

    private void createField()
    {
        JPanel p = new JPanel();
        p.setLayout(null);
        JLabel lbl1 = new JLabel("検索するファイル名");
        lbl1.setBounds(10, 10, 114, 24);
        HistoryText h1 = new  HistoryText(32);
        h1.loadHistory("FindDialog.source");
        filePtn = h1.text;
        h1.setBounds(124, 10, 290, 24);

        JLabel lbl3 = new JLabel("検索するフォルダ");
        lbl3.setBounds(10, 36, 114, 24);
        HistoryCombo h3 = new  HistoryCombo(32);
        h3.loadHistory("FindDialog.folder");
        setSrcFolder(h3);
        folder  = h3.text;
        h3.setBounds(124, 36, 290, 24);

        refBtn = new JButton("...");
        refBtn.setBounds(413, 36, 24, 24);

        p.add(lbl1);
        p.add(h1);
        p.add(lbl3);
        p.add(h3);
        p.add(refBtn);

        caseChk = new JCheckBox("大文字と小文字を区別しない");
        caseChk.setBounds(10, 60, 192, 24);
        wordChk = new JCheckBox("単語単位での検索");
        wordChk.setBounds(10, 60 + 24 + 4, 172, 24);
        regChk  = new JCheckBox("正規表現");
        regChk.setBounds(10, 60 + 24 + 24 + 8, 100, 24);

        searchBtn    = new JButton("検索");
        searchBtn.setBounds(300, 60 + 24 + 4, 110, 24);
        cancelBtn    = new JButton("キャンセル");
        cancelBtn.setBounds(300, 60 + 24 + 24 + 8, 110, 24);

        JLabel lbl4 = new JLabel("検索数の上限");
        lbl4.setBounds(120,     60 + 24 + 24 + 8, 100, 24);
        maxCombo = new JComboBox(comboItems);
        maxCombo.setBounds(200, 60 + 24 + 24 + 8, 60, 24);
        maxCombo.setSelectedIndex(0);
        p.add(caseChk);
        p.add(wordChk);
        p.add(regChk);

        p.add(lbl4);
        p.add(maxCombo);

        p.add(searchBtn);
        p.add(cancelBtn);

        getContentPane().add(p, BorderLayout.CENTER);

        statusLabel = new JLabel("　　　　　");
        statusLabel.setFont(EditorConfig.font);
        statusLabel.setHorizontalAlignment(SwingConstants.LEFT);
        statusLabel.setBorder(new BevelBorder(BevelBorder.LOWERED));
        statusLabel.setPreferredSize(new Dimension(448, 24));
        getContentPane().add(statusLabel, BorderLayout.SOUTH);

    }

    private void setSrcFolder(HistoryCombo h3)
    {
        if (resultAct == null)
            return;

        File home = resultAct.getSrcFolder();
        setSrcFolderSub(home, h3);
    }

    private void setSrcFolderSub(File f, HistoryCombo h3)
    {
        if (f.isDirectory())
        {
            h3.combo.addItem(f.getPath());
            for(File c : f.listFiles())
            {
                setSrcFolderSub(c, h3);
            }
        }
    }

    private void setStatusBar(final String s)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                statusLabel.setText(s);
            }
        });
    }

    private void setListener()
    {
        refBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               openChooser();
            }
        });

        searchBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               search();
            }
        });

        cancelBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               cancel();
            }
        });
    }

    File selDir = null;

    private void openChooser()
    {
        JFileChooser fc;
        if (selDir != null)
            fc = new JFileChooser(selDir);
        else
            fc = new JFileChooser();

        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setMultiSelectionEnabled(false);

        int selected;
        selected = fc.showOpenDialog(this);
        if (selected != JFileChooser.APPROVE_OPTION)
            return;

        try
        {
            File file = fc.getSelectedFile();
            folder.setText(file.getPath().replace("\\","/"));
            selDir = file.getParentFile();
        }
        catch(Exception ex){  }
    }

    private String getFileTargetText()
    {
        String d = filePtn.getText();
        if (regChk.isSelected())
        {
            return d;
        }
        String [] metaChars = {"\\",".","(",")","+","$","^","[","]",":","{","}",">","<","=",","};
        for(String c : metaChars)
        {
            d = d.replace(c, "\\" + c);
        }

        if (wordChk.isSelected())
            d = "\\b" + d + "\\b";

        if (caseChk.isSelected())
            d = d.toLowerCase();
        return d;
    }

    public void search()
    {
        if (filePtn.getText().isEmpty())
        {
            setStatusBar("検索するファイル名が未入力です.");
            return;
        }
        if (folder.getText().isEmpty())
        {
            setStatusBar("対象フォルダが未入力です.");
            return;
        }

        topDir = new File(folder.getText());
        if (!topDir.exists())
        {
            setStatusBar("対象フォルダが存在しません.");
            return;
        }

        targetText = getFileTargetText();

        try
        {
            regPattn  = Pattern.compile(targetText);
            limmit = Integer.parseInt((String)maxCombo.getSelectedItem());
        }
        catch(Exception ex)
        {
            setStatusBar("ファイル名の正規表現が間違っています.");
            return;
        }

        Runnable r = new Runnable()
        {
            public void run()
            {
                canceled = false;
                findThread();
            }
        };
        Thread th = new Thread(r);
        th.start();
    }

    public void cancel()
    {
        canceled = true;
    }

    boolean canceled = false;

    String  targetText = "";
    String  fileText   = "";
    File    topDir     = null;
    Pattern     regPattn  = null;
    int limmit = 100;
    int resCount = 0;

    ArrayList<File> resList  = new ArrayList<File>();

    public void findThread()
    {
        int c = 0;
        try
        {
            setStatusBar("検索を開始します.");
            resList.clear();
            resCount = 0;

            findDir(topDir);

            ArrayList<String[]> result = new ArrayList<String[]>();
            for(File f : resList)
            {
                result.add(new String[] { f.getPath().replace("\\","/"),
                                          "", ""  });
                ++c;
            }
            if (c > 0)
            {
                String title = String.format("%s (%s) 結果 %d 件",
                                             topDir.getPath().replace("\\", "/"),
                                             filePtn.getText(), 
                                             c);

                if (resultAct != null)
                    resultAct.addResult(SearchType.FIND, title,  result);
            }
        }
        catch(Exception ex)
        {
            return;
        }
        setStatusBar(String.format("検索が終了しました.(結果 %d 件)", c));
    }

    public void findDir(File dir)
    {
       if (canceled)
           return;

        for(File f : dir.listFiles())
        {
            if (f.isFile())
            {
                Matcher m = regPattn.matcher(f.getName());
                if (m.matches())
                {
                    resList.add(f);
                }
            }
            else if (f.isDirectory())
            {
                setStatusBar("処理中:" + f.getPath().replace("\\","/"));
                findDir(f);
            }
            if (canceled)
                return;
            if (resCount >= limmit)
                return;
        }
    }

}


