//============================================================================
//
//    Java Text Editor library
//
//    SearchPanel.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.awt.event.*;

import java.util.regex.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;

public class SearchPanel extends JPanel
{
    private JTextField    src    = null;
    private JTextField    dst    = null;

    private JCheckBox      wordChk   = null;
    private JCheckBox      caseChk   = null;
    private JCheckBox      regChk   = null;

    private JCheckBox      reverse   = null;
    private JCheckBox      all      = null;

    private JButton        sBtn    = null;
    private JButton        tBtn    = null;

    private JTextComponent    text     = null;

    public SearchPanel(JTextComponent owner)
    {
        text = owner;

        setLayout(null);

        createField();

        setListener();
    }

    private void createField()
    {
        JLabel lbl1 = new JLabel("検索する文字列");
        lbl1.setBounds(10, 10, 100, 24);
        HistoryText h1 = new  HistoryText(32);
        h1.loadHistory("SearchDialog.source");
        src = h1.text;
        src.setToolTipText("インクリメンタルサーチ.");
        h1.setBounds(114, 10, 300, 24);

        JLabel lbl2 = new JLabel("置換後の文字列");
        lbl2.setBounds(10, 36, 100, 24);
        HistoryText h2 = new  HistoryText(32);
        h2.loadHistory("SearchDialog.desert");
        dst  = h2.text;
        h2.setBounds(114, 36, 300, 24);
        add(lbl1);
        add(h1);
        add(lbl2);
        add(h2);

        caseChk = new JCheckBox("大文字と小文字を区別しない");
        caseChk.setBounds(10, 68, 172, 24);
        wordChk = new JCheckBox("単語単位での検索");
        wordChk.setBounds(10, 68 + 24 + 4, 172, 24);
        regChk  = new JCheckBox("正規表現");
        regChk.setBounds(10, 68 + 24 + 24 + 8, 172, 24);

        reverse = new JCheckBox("上へ");
        reverse.setBounds(250, 68, 60, 24);
        all     = new JCheckBox("全て");
        all.setBounds(250, 68 + 24 + 4, 60, 24);

        sBtn    = new JButton("検索");
        sBtn.setBounds(310, 68, 110, 24);
        tBtn    = new JButton("置換");
        tBtn.setBounds(310, 68 + 24 + 4, 110, 24);

        add(caseChk);
        add(wordChk);
        add(regChk);

        add(reverse);
        add(all);
        add(sBtn);
        add(tBtn);
    }

    private void setListener()
    {
        src.getDocument().addDocumentListener(new DocumentListener()
        {
            public void changedUpdate(DocumentEvent de)
            {
                search();
            }
            public void insertUpdate(DocumentEvent de)
            {
                search();
            }
            public void removeUpdate(DocumentEvent de)
            {
                search();
            }
        });

        src.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               search();
            }
        });

        sBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               search();
            }
        });

        tBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               translate();
            }
        });

        this.addFocusListener(new FocusListener()
        {
             public void focusGained(FocusEvent e)
             {
                text.getCaret().setSelectionVisible(true);
             }

             public void focusLost(FocusEvent e)
             {
             }
        });
        
    }

    public void translate()
    {
        try
        {
            if (all.isSelected())
            {
                String s = src.getText();
                String d = dst.getText();

                String  t = text.getText();
                text.setText(t.replace(s, d));
            }
            else
            {
                String  s = text.getSelectedText();

                if (s != null && !s.equals(""))
                {
                    String d = dst.getText();
                    text.replaceSelection(d);
                    text.getCaret().setSelectionVisible(true);
                }
            }
        }
        catch(Exception ex) {}
    }

    public void setReverse(boolean onoff)
    {
        reverse.setSelected(onoff);
    }

    private String getSearchTargetText()
    {
        String d = src.getText();
        if (regChk.isSelected())
            return d;

        String [] metaChars = {"\\",".","(",")","+","$","^","[","]",":","{","}",">","<","=",","};
        for(String c : metaChars)
        {
            d = d.replace(c, "\\" + c);
        }

        if (wordChk.isSelected())
            d = "\\b" + d + "\\b";

        if (caseChk.isSelected())
            d = d.toLowerCase();

        return d;
    }

    public void search()
    {
        search(null, reverse.isSelected());
    }

    public void search(String sel)
    {
        search(sel, reverse.isSelected());
    }

    public void search(String sel, boolean rev)
    {
        if (sel != null && !sel.equals(""))
        {
            if (!src.getText().equals(sel))
            {
                src.setText(sel);
                return;
            }
        }

        String d = getSearchTargetText();
        try
        {
            String s = SourceToolBar.delCR(text.getText());

            if (caseChk.isSelected())
                s = s.toLowerCase();

            int ci = text.getCaretPosition();
            int ed = text.getSelectionEnd();
            int st = text.getSelectionStart();
            Pattern p = Pattern.compile(d);

            int i = ci;
            if (rev)
            {
                boolean rc = seekRSub(p, s, d, i);
                if (rc)
                {
                    if (st != text.getSelectionStart())
                       return;
                    rc = seekRSub(p, s, d, st);
                }
            }
            else
            {
                if (ci == ed)
                    i = st;
                boolean rc = seekSub(p, s, d, i);
                if (rc)
                {
                   if (ed != text.getSelectionEnd())
                       return;
                    rc = seekSub(p, s, d, ci);
                }
            }
        }
        catch(Exception ex)
        {
        }
    }

    private boolean seekSub(Pattern pattn, String s, String d, int start)
    {
        String o = s.substring(start, s.length());

        Matcher m = pattn.matcher(o);
        boolean b = m.find();

        if (b == false)
        {
            if (start != 0)
                return seekSub(pattn, s, d, 0);
            return false;
        }
        MatchResult res = m.toMatchResult();
        text.select(start + res.start(), start + res.end());
        text.getCaret().setSelectionVisible(true);
        return true;
    }

    private boolean seekRSub(Pattern pattn, String s, String d, int start)
    {
        String o = s.substring(0, start);

        Matcher m = pattn.matcher(o);
        MatchResult res = null;

        while(m.find() == true)
            res = m.toMatchResult();

        if (res == null)
        {
            if (start < s.length())
                return seekRSub(pattn, s, d, s.length());

            return false;
        }
        text.select(res.start(),  res.end());
        text.getCaret().setSelectionVisible(true);
        return true;
    }

    public void setTranseFocus(String sel)
    {
        if (sel != null && !sel.equals(""))
        {
           src.setText(sel);
        }

        dst.requestFocus();
    }

     public void allOff()
     {
         all.setSelected(false);
     }
}
