//============================================================================
//
//    Java Text Editor library
//
//    HighLightControler.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.event.*;
import java.awt.*;
import java.io.*;

import javax.swing.*;
import javax.swing.text.*;
import java.util.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;

import static text.editor.Kit.*;

public class HighLightControler
{
    static class HLEntry
    {
        public String key;
        public String fonttype="";
        public String bgcolor="";
        public String fgcolor="";
        public String text="";

        public HLEntry() {}

        public HLEntry(String key)
        {
           this.key = key;
        }

        public String toString()
        {
            StringBuilder sb = new StringBuilder();

            sb.append(":key=" + key);

            if (fonttype == null)
               sb.append(":fonttype=null");
            else
               sb.append(":fonttype=" + fonttype);

            if (bgcolor == null)
               sb.append(":bgcolor=null");
            else
               sb.append(":bgcolor=" + bgcolor);

            if (fgcolor == null)
               sb.append(":fgcolor=null");
            else
               sb.append(":fgcolor=" + fgcolor);

            return sb.toString();
        }
    }

    static Hashtable<String, Integer> postFix = new Hashtable<String, Integer>();
    static Hashtable<Integer, Object> extMap  = null;

    static void initialize()
    {
        extMap = new Hashtable<Integer, Object>();
        try
        {
            File dir = new File(Kit.HOME + "/config/hilight");
            int c = 0;
            for(File f : dir.listFiles())
            {
                if (f.getName().endsWith(".csv"))
                {
                    Hashtable<String, HLEntry> tbl = new Hashtable<String, HLEntry>();
                    String [] exts = load(f, tbl);
                    if (exts != null)
                    {
                        extMap.put(c, tbl);
                        for(int i = 1 ; i < exts.length; ++i)
                        {
                            postFix.put(exts[i].trim().toUpperCase(), c);
                        }
                        ++c;
                    }
                }
            }
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
    }

    public static Object getExtEntry(String ext)
    {
        if (postFix.containsKey(ext.toUpperCase()))
        {
            Integer i = postFix.get(ext.toUpperCase());
            return extMap.get(i);
        }
        return null;
    }

    public static boolean containsExtEntry(String ext)
    {
        return postFix.containsKey(ext.toUpperCase());
    }

    Hashtable<String, HLEntry> tbl = null;
    HighLightedDocument   doc = null;
    JTextPane             text = null;

    public String idStartChar = "";
    public String idNextChar  = "";
    public String idEndChar   = "";
    public String cnstChar    = "";
    public String graphKeyChar = "";
    public String onelineComChar = "";

    public int comType = 0; // 0:C , 1:xml  2:oneline 
    public String kakko = "";
    public String kkoka = "";

    public HighLightControler(HighLightedDocument doc,
                                 JTextPane text, Object t)
    {
         this.doc  = doc;
         this.text = text;
         this.tbl = (Hashtable<String, HLEntry>)t;

         if (tbl.containsKey("id_start"))
             this.idStartChar = tbl.get("id_start").text;

         if (tbl.containsKey("id_next"))
             this.idNextChar = tbl.get("id_next").text;

         if (tbl.containsKey("id_end"))
             this.idEndChar = tbl.get("id_end").text;

         if (tbl.containsKey("const_char"))
             this.cnstChar = tbl.get("const_char").text;

         if (tbl.containsKey("kakko"))
             this.kakko = tbl.get("kakko").text;

         if (tbl.containsKey("kkoka"))
             this.kkoka = tbl.get("kkoka").text;

         if (tbl.containsKey("comment_type"))
         {
             String tt = tbl.get("comment_type").text;
             this.comType = 0;
             if (tt.equals("MarkUp"))
                 this.comType = 1;
             if (tt.equals("Oneline"))
                 this.comType = 2;
         }

         if (tbl.containsKey("graph_key"))
             this.graphKeyChar = tbl.get("graph_key").text;

         if (tbl.containsKey("oneline_char"))
             this.onelineComChar = tbl.get("oneline_char").text;
    }

    public void setStyles() throws Exception
    {
        set();
    }

    public  void resetStyles()
    {
        Enumeration<String> e = tbl.keys();
        while (e.hasMoreElements())
        {
            String k = e.nextElement();
            doc.removeStyle(k);
        }

        doc.removeStyle("enter");
        doc.removeStyle("space");

        set();
    }

    private void set()
    {
        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
        doc.addStyle("regular", def);

        Font font = EditorConfig.font;
        StyleConstants.setFontFamily(def, font.getFamily());
        StyleConstants.setFontSize  (def, font.getSize());
        StyleConstants.setBold      (def, font.isBold());
        StyleConstants.setItalic    (def, font.isItalic());

        StyleConstants.setBackground(def, EditorConfig.bgColor);
        StyleConstants.setForeground(def, EditorConfig.fgColor);

        Enumeration<String> e = tbl.keys();
        while (e.hasMoreElements())
        {
            String k = e.nextElement();
            HLEntry entry = tbl.get(k);

            addStyle(entry, def);
        }

        addStyle(new HLEntry("space"), def);
        addStyle(new HLEntry("enter"), def);

        SimpleAttributeSet attrs = new SimpleAttributeSet();

        FontMetrics fm = text.getFontMetrics(EditorConfig.font);
        int size = EditorConfig.tabSize;
        int tabLength = fm.charWidth(' ') * size;
        TabStop[] tabs = new TabStop[100];
        for(int j=0;j<tabs.length;j++) 
        {
            tabs[j] = new TabStop((j+1)*tabLength);
        }
        TabSet tabSet = new TabSet(tabs);
        StyleConstants.setTabSet(attrs, tabSet);

        doc.setParagraphAttributes(0, doc.getLength(), attrs, false);
    }

    private Style addStyle(HLEntry e, Style regular)
    {
        try
        {
            Style s = doc.addStyle(e.key, regular);

            if (e.fonttype != null)
            {
                StyleConstants.setBold  (s, e.fonttype.equals("bold"));
                StyleConstants.setItalic(s, e.fonttype.equals("italic"));
            }

            if (e.bgcolor == null || e.bgcolor.length() == 0)
                ;
            else
                StyleConstants.setBackground(s, Kit.cnvColorRGB(e.bgcolor));

            if (e.fgcolor == null || e.fgcolor.length() == 0)
                ;
            else
                StyleConstants.setForeground(s, Kit.cnvColorRGB(e.fgcolor));

            return s;
        }
        catch(Exception ex)
        {
        }
        return null;
    }

    private static String [] load(File path, Hashtable<String, HLEntry> tbl)
    {
        String [] extmap = null;
        BufferedReader path_rd= null;
        try
        {
            path_rd= new BufferedReader(new FileReader(path));
            String line;
            while((line = path_rd.readLine()) != null)
            {
                if (line.startsWith("#") || line.isEmpty())
                    continue;

                String [] col = line.split(",");
                if (col[0].trim().equals("extmap"))
                {
                    extmap = col;
                    continue;
                }
                addEntry(col, tbl);
            }
        }
        catch(Exception e)
        {
        }
        finally
        {
            if (path_rd != null)
                try { path_rd.close(); }
                catch(Exception ex){}
        }
        return extmap;
    }

    private static void addEntry(String [] cl, Hashtable<String, HLEntry> tbl)
    {
        String k = cl[0].trim();
        if (k.equals("id_start") || k.equals("id_next")
         || k.equals("id_end") || k.equals("const_char")
         || k.equals("graph_key") || k.equals("comment_type") )
        {
            HLEntry e = new HLEntry();
            e.key = k;
            e.text = cl[1].trim();
            tbl.put(e.key, e);
            if (k.equals("comment_type") && cl.length == 3)
            {
                HLEntry e2 = new HLEntry();
                e2.key = "oneline_char";
                e2.text = cl[2].trim();
                tbl.put("oneline_char", e2);
            }
        }
        else if (k.equals("comment") || k.equals("constant"))
        {
            HLEntry e = new HLEntry();
            e.key = k;
            e.fgcolor = cl[1].trim();
            tbl.put(e.key, e);
        }
        else if (k.equals("keyword"))
        {
            String s = cl[1].trim();

            for(int i = 2; i < cl.length; ++i)
            {
                HLEntry e = new HLEntry();
                e.key = cl[i].trim();
                if (s.charAt(0) == 'B')
                {
                    e.fgcolor = s.substring(1);
                    e.fonttype = "bold";
                }
                else if (s.charAt(0) == 'I')
                {
                    e.fgcolor = s.substring(1);
                    e.fonttype = "italic";
                }
                else
                {
                    e.fgcolor = s;
                }
                tbl.put(e.key, e);
            }
        }
    }

    private static org.w3c.dom.Document loadXML(File f)
    {
        try
        {
            DocumentBuilder b = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            return b.parse(f);
        }
        catch(Exception e){}
        return null;
    }

    private static void createHash(org.w3c.dom.Document xmldoc, Hashtable<String, HLEntry> tbl)
    {
        org.w3c.dom.Element elem = xmldoc.getDocumentElement();

        if (!elem.hasChildNodes())
            return;

        NodeList nl  = elem.getChildNodes();
        for (int i = 0; i < nl.getLength(); ++i)
        {
            Node n = nl.item(i);

            NamedNodeMap map = n.getAttributes();
            if (map != null)
            {
                HLEntry e = new HLEntry();
                e.key   = getMapItem(map, "key");

                if (e.key != null)
                {
                    e.fonttype = getMapItem(map, "fonttype");
                    e.bgcolor  = getMapItem(map, "bgcolor");
                    e.fgcolor  = getMapItem(map, "fgcolor");

                    Node nn = n.getFirstChild();
                    if (nn != null)
                        e.text = nn.getTextContent();

                    tbl.put(e.key, e);
                }
            }
        }
    }

    private static String getMapItem(NamedNodeMap map, String key)
    {
        Node n = map.getNamedItem(key);
        if (n == null)
           return null;

        return n.getNodeValue();
    }

    public void dump()
    {
        Enumeration<String> e = tbl.keys();
        while (e.hasMoreElements())
        {
            String k = e.nextElement();
            HLEntry entry = tbl.get(k);

            System.out.println(entry.toString());
        }
    }
}

