//============================================================================
//
//    Java Text Editor library
//
//    HistoryText.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import java.util.*;

import static text.editor.Kit.*;

public class HistoryText extends JPanel
{
    static  final int MAX_HISTORY = 20;
    static  ImageIcon icon = getIconImage("History.GIF");

    private JLabel               btn;
    public  JTextField          text;
    private  LinkedList<String> hist = null;
    private  JList               list;
    private  DefaultListModel   model;
    private  JScrollPane        scroll;

    public HistoryText(int col)
    {
        super();

        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        setBorder(new EmptyBorder(1,1,1,1));

        btn  = new JLabel(icon);
        text = new JTextField(col);

        btn.setToolTipText("クリック/Downキーで履歴が選択できます.");

        add(btn);
        add(text);

        model = new DefaultListModel();

        list  = new JList(model);
        list.setBorder(new EmptyBorder(new Insets(2, 4, 2, 4)));
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        scroll =
            new JScrollPane(list,
                ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        text.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent e)
            {
                int c =  e.getKeyCode();
                if (c == KeyEvent.VK_ENTER)
                    addHistory();
                else if (c == KeyEvent.VK_DOWN)
                    showPopup();
            }
        });

        btn.addMouseListener(new MouseAdapter()
        {
            public void mouseClicked(MouseEvent e)
            {
                showPopup();
            }
        });

        list.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent e)
            {
                int c =  e.getKeyCode();
                if (c == KeyEvent.VK_ENTER)
                    valueChanged();
                else if (c ==  KeyEvent.VK_DELETE)
                    delHistory();
            }
        });

        list.addMouseListener(new MouseAdapter()
        {
            public void mouseClicked(MouseEvent e)
            {
                valueChanged();
            }
        });
    }

    public void loadHistory(String historyKey)
    {
        hist = UserHistory.get(historyKey);
    }

    public void addHistory()
    {
        String ss = text.getText().trim();
        if (!ss.equals("") && !hist.contains(ss))
        {
            hist.addFirst(ss);
            if (hist.size() > MAX_HISTORY)
               hist.removeLast();
        }
    }

    public void delHistory()
    {
        String str = (String)list.getSelectedValue();
        if (str == null)
            return;

        if (hist.contains(str))
        {
            hist.remove(str);
            model.removeElement(str);
        }
    }

    public void valueChanged()
    {
        String str = (String)list.getSelectedValue();

        if (str != null)
        {
            text.setText(str);
            for (ActionListener act : text.getActionListeners())
                act.actionPerformed(null);
        }

        pop.setVisible(false);
    }

    JPopupMenu pop = null;

    private void showPopup()
    {
        if (hist.size() == 0)
            return;

        Point los = btn.getLocation();
        Dimension dim = btn.getSize();
        int popX = los.x;
        int popY = los.y + dim.height;

        pop = new JPopupMenu();
        pop.add(scroll);

        model.clear();
        for (Object m : hist)
        {
            model.addElement(m);
        }

        pop.show(this, popX, popY);
        list.setSelectedIndex(0);
        list.requestFocus();
    }
}

