//============================================================================
//
//    Java Text Editor library
//
//    InputDialog.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;
import javax.swing.border.*;

public class InputDialog extends JDialog 
{
    public  boolean    result = false;
    public  JComponent [] fields = null;
    private JButton ok = null;
    private JButton ng = null;

    boolean [] chks;

    public InputDialog(Frame owner, String title, Image icon,
                       String [] explain, String [] initValue ,
                       boolean [] mustbe)
    {
        super(owner, title, true);
        setIconImage(icon);

        this.chks = mustbe;
        JPanel p = new JPanel();
        p.setLayout(null);
        fields = new JComponent[explain.length];
        int x = 10;
        int y = 10;

        for(int i = 0; i < explain.length;++i)
        {
            x = 23;
            // explainに＠がついている場合はCheckBox
            if (explain[i].startsWith("@"))
            {
                JCheckBox chk = new JCheckBox(explain[i].substring(1));
                if (initValue[i].equals("True") 
                || initValue[i].equals("true"))
                    chk.setSelected(true);
                chk.setBounds(x,y, 300, 23);
                fields[i] = chk;
                p.add( fields[i]);
                x += 300;
            }
            else
            {
                JLabel lbl = new JLabel(explain[i]);
                lbl.setBounds(x,y, 100, 23);
                x += 100;
                JTextField txt = new JTextField(initValue[i]);
                txt.setBounds(x,y, 200, 23);
                txt.setCaretPosition(0);
                fields[i] = txt;
                p.add(lbl);
                p.add( fields[i]);
                x += 200;
            }
            y += 26;
        }

        p.setPreferredSize(new Dimension(x+10, y+10));
        getContentPane().add(p, BorderLayout.CENTER);
        getContentPane().add(createBtns(), BorderLayout.SOUTH);

        setListener();

        pack();

        setSize(x+10, y+16+50);
        setResizable(false);

        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(size.width /2 - getSize().width /2,
                    size.height/2 - getSize().height/2);
    }

    private JComponent createBtns()
    {
        JPanel p = new JPanel();

        ok = new JButton("了解");
        ng = new JButton("取消");

        p.add(Box.createHorizontalGlue());
        p.add(ok);
        p.add(ng);
        p.add(Box.createHorizontalGlue());

        return p;
    }

    private void setListener()
    {
        ActionListener act =  new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                ending(e);
            }
        };

        ok.addActionListener(act);
        ng.addActionListener(act);
    }

    public void ending(ActionEvent e)
    {
        JButton s = (JButton)e.getSource();

        if (s.getText().equals("了解"))
        {
            if (!emptyCheck())
            {
                return;
            }
            result = true;
        }
        dispose();
    }

    public boolean emptyCheck()
    {
        for(int i=0;i<fields.length;++i)
        {
            if (chks[i] && fields[i] instanceof JTextField)
            {
                if (((JTextField)fields[i]).getText().length() == 0)
                {
                    Kit.messageDialog("必須項目が入力されていません.");
                    fields[i].requestFocus();
                    return false;
                }
            }
        }
        return true;
    }

    public String[] getResult()
    {
        String [] res = new String[fields.length];
        for(int i = 0; i < fields.length;++i)
        {
            if (fields[i] instanceof JTextField)
            {
                res[i] = ((JTextField)fields[i]).getText();
            }
            else
            {
                res[i] = "" + ((JCheckBox)fields[i]).isSelected();
            }
        }
        return res;
    }

}
