//============================================================================
//
//    Java Text Editor library
//
//    KitManager.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.io.*;
import java.util.*;
import javax.swing.*;

public class KitManager 
{
    static final ArrayList<TxtEditorTabPanel> editors  = new ArrayList<TxtEditorTabPanel>();  
    static final ArrayList<HtmlPanel>          htmls   = new ArrayList<HtmlPanel>();  

    public static void clear()
    {
        editors.clear();
        htmls.clear();
    }

    public static void addEditor(TxtEditorTabPanel edt)
    {
        if (!editors.contains(edt))
            editors.add(edt);
    }

    public static void removeEditor(TxtEditorTabPanel edt)
    {
        if (editors.contains(edt))
            editors.remove(edt);
    }

    public static TxtEditorTabPanel getTextEditor(String filePath)
    {
        String fp = filePath.replace("\\", "/");
        for(TxtEditorTabPanel ed : editors)
        {
            if (ed.getFilePath().equals(fp)) 
            {
                return ed;
            }
        }
        return null;
    }

    public static boolean containsNewFile(String nm)
    {
        for(TxtEditorTabPanel ed : editors)
        {
            if (ed.mode == 1
             && ed.getPanelTitle().equals(nm)) 
            {
                return true;
            }
        }
        return false;
    }

    public static void refresh()
    {
        for(TxtEditorTabPanel ed : editors)
        {
            ed.refresh();
        }
    }

    public static void saveAll()
    {
        for(TxtEditorTabPanel ed : editors)
        {
            if (ed.isEdited) 
            {
                if (ed.writeCheckTimeStamp())
                    ed.saveSilent();
            }
        }
   }

    public static void closeAll()
    {
        closeAllEditor();
        closeAllHtml();
        clear();
    }

    private static void closeAllEditor()
    {
        ArrayList<TxtEditorTabPanel> dels = new ArrayList<TxtEditorTabPanel>();
        for(TxtEditorTabPanel ed : editors)
        {
            if (Kit.isDockTab(ed))
            {
                Kit.removeTab(ed);
            }
            else
            {
                dels.add(ed);
            }
        }

        for(TxtEditorTabPanel ed : dels)
        {
            if (ed.tabClose())
            {
               Kit.closeFrame(ed);
            }
        }
    }

    private static void closeAllHtml()
    {
        ArrayList<HtmlPanel> dels = new ArrayList<HtmlPanel>();

        for(HtmlPanel pg : htmls)
        {
            if (Kit.isDockTab(pg))
            {
                Kit.removeTab(pg);
            }
            else
            {
                dels.add(pg);
            }
        }
        for(HtmlPanel pg : htmls)
        {
            if (pg.tabClose())
            {
                Kit.closeFrame(pg);
            }
        }
    }
}
