//============================================================================
//
//    Java Text Editor library
//
//    MultiClipboardManager.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;


public class MultiClipboardManager
{
    static final int ITEM_MAX = 100;
    static final LinkedList<String>  clipList = new LinkedList<String> ();

    public static LinkedList<String>  getClipList()
    {
        return clipList;
    }

    static ActionListener clipChanged = null;

    public static void setClipChangeAction(ActionListener act)
    {
        clipChanged = act;
    }

    public static void initialize()
    {
        Toolkit kit = Toolkit.getDefaultToolkit();
        Clipboard clip = kit.getSystemClipboard();

        clip.addFlavorListener(new FlavorListener()
        {
            public void flavorsChanged(FlavorEvent e)
            {
                pushClip();
            }
        });
    }

    public static void clear()
    {
        clipList.clear();
        onClipboardChanged();
    }

    public static void push(String val)
    {
        if (val == null)
            return;
        addList(val);

        setClipboardString(val);
    }

    public static void addList(String val)
    {
        if (val == null)
            return;
        int idx = clipList.indexOf(val);
        if (idx != -1)
            clipList.remove(idx);
        clipList.addFirst(val);

        while (clipList.size() > ITEM_MAX)
            clipList.removeLast();
        onClipboardChanged();
    }

    public static void pushClip()
    {
        push(getClipboardString());
    }

    public static void remove(int idx)
    {
        if (idx < clipList.size())
        {
            clipList.remove(idx);
            onClipboardChanged();
        }
    }

    public static void setClipbord(int idx)
    {
        if (idx < clipList.size())
        {
            String s = clipList.get(idx);
            setClipboardString(s);
        }
    }

    private static void onClipboardChanged()
    {
        if (clipChanged != null)
        {
            clipChanged.actionPerformed(null);
        }
    }

    public static String getClipboardString()
    {
        Toolkit kit = Toolkit.getDefaultToolkit();
        Clipboard clip = kit.getSystemClipboard();

        try
        {
            return (String) clip.getData(DataFlavor.stringFlavor);
        }
        catch (UnsupportedFlavorException e)
        {
            return null;
        }
        catch (IOException e)
        {
            return null;
        }
        catch (Exception e)
        {
            return null;
        }
    }

    public static void setClipboardString(String str) 
    {
        Toolkit kit = Toolkit.getDefaultToolkit();
        Clipboard clip = kit.getSystemClipboard();

        StringSelection ss = new StringSelection(str);
        clip.setContents(ss, ss);
    }
}
