//============================================================================
//
//    Java Text Editor library
//
//    UserHistory.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import java.util.*;
import java.io.*;

public class UserHistory
{
    private static String FILE; 

    static Hashtable<String, LinkedList<String> > historyHash
                                 = new  Hashtable<String, LinkedList<String> >();

    static void initilaize()
    {
        FILE = Kit.HOME + "/config/history.ini"; 
    }

    static LinkedList<String> get(String key)
    {
        if (!historyHash.containsKey(key))
        {
            LinkedList l = new LinkedList();
            historyHash.put(key, l);
            return l;
        }
        return historyHash.get(key);
    }

    public static void load()
    {
        File file = new File(FILE);
        if (!file.exists())
            return;

        BufferedReader br = null;
        try
        {
            br = new BufferedReader(new InputStreamReader(new FileInputStream(file), "UTF-8"));

            String line;
            while ((line = br.readLine()) != null)
            {
                int i = line.indexOf(':');
                String k = line.substring(0, i);
                String v = line.substring(i + 1, line.length());

                LinkedList<String> l = get(k);
                l.addFirst(v);
            }
        }
        catch(IOException ex)
        {
            ex.printStackTrace();
        } 
        finally
        {
            if(br != null)
            {
                try { br.close(); }
                catch(Exception e) {}
            }
        }
    }

    public static  void save()
    {
        File file = new File(FILE);
        OutputStream os = null;
        Writer w = null;
        BufferedWriter bw = null;
        try
        {
            os = new FileOutputStream(file);
            w  = new OutputStreamWriter(os, "UTF-8");
            bw = new BufferedWriter(w);

            // format KEY:VALUE
            for (String k : historyHash.keySet())
            {
                LinkedList<String> l = historyHash.get(k);
                int max = l.size();
                for (int i = 0;i < max; ++i)
                {
                    bw.write(String.format("%s:%s", k, l.get(i)));
                    bw.newLine();
                }
            }
        }
        catch(IOException ex)
        {
            ex.printStackTrace();
        } 
        finally
        {
            if (bw != null) try { bw.close(); } catch (IOException e) {}
            if (w  != null) try { w .close(); } catch (IOException e) {}
            if (os != null) try { os.close(); } catch (IOException e) {}
        }
    }
}

