//============================================================================
//
//    Java Text Editor library
//
//    KeyBindings.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================package text.editor;
package text.editor;

import java.awt.*;
import java.awt.event.*;

import java.util.*;
import java.util.concurrent.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.undo.*;
import java.beans.PropertyChangeListener;

public class KeyBindings
{
    static char []  bindCode    = {'0','1','2','3','4','5','6','7','8','9',
                                   'A','B','C','D','E','F','G','H','I','J','K','L','M',
                                   'N','O','P','Q','R','S','T','U','V','W','X','Y','Z'};

    static String []  keyStr  = { "C-",  "CShft-",  "CAlt-"  };

    static int    []  keyMask = {
                                    InputEvent.CTRL_DOWN_MASK,
                                    InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK,
                                    InputEvent.CTRL_DOWN_MASK | InputEvent.ALT_DOWN_MASK
                                 };

    JEditorPane   text;

    public KeyBindings(SourceToolBar tools)
    {
        this.text  = tools.text;
    }

    public void keyBind()
    {
        Keymap km = text.getKeymap();

        KeyStroke ks5 = KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0);
        km.removeKeyStrokeBinding(ks5);
        km.addActionForKeyStroke(ks5, new TabKeyAction());

        KeyStroke ks6 = KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_MASK);
        km.removeKeyStrokeBinding(ks6);
        km.addActionForKeyStroke(ks6, new ShiftTabKeyAction());

        KeyStroke ks9 = KeyStroke.getKeyStroke("ctrl Z");
        km.removeKeyStrokeBinding(ks9);
        km.addActionForKeyStroke(ks9, new CtrlZAction());

        KeyStroke ks10 = KeyStroke.getKeyStroke("ctrl Y");
        km.removeKeyStrokeBinding(ks10);
        km.addActionForKeyStroke(ks10, new CtrlYAction());

        KeyStroke ks11 = KeyStroke.getKeyStroke("ctrl A");
        km.removeKeyStrokeBinding(ks11);
        km.addActionForKeyStroke(ks11, new CtrlAAction());

        KeyStroke ks12 = KeyStroke.getKeyStroke("ctrl shift DELETE");
        km.removeKeyStrokeBinding(ks12);
        km.addActionForKeyStroke(ks12, new CtrlShiftDAction());

        KeyStroke ks14 = KeyStroke.getKeyStroke("alt I");
        km.removeKeyStrokeBinding(ks14);
        km.addActionForKeyStroke(ks14, new FocusIncTextAction());

        actionMap();
    }

    void actionMap()
    {
        ActionMap am = text.getActionMap();
        am.put(DefaultEditorKit.selectWordAction, noselAction);
        am.put(DefaultEditorKit.selectLineAction, noselAction);
    }

    static final Action noselAction = new AbstractAction()
    {
        @Override public void actionPerformed(ActionEvent e)
        {
        }
    };

    ///////////////////////////////////////////////////////////////////////////

    static class CtrlZAction  extends BindsAction 
    {
        public CtrlZAction()
        {
           super("CtrlZAction");
        }
        public void actionPerformed(ActionEvent e)
        {
            final SourceToolBar tool = getTools(e);
            if (tool == null)
                return;
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    tool.undo();
                }
            });
        }
    }

    static class CtrlYAction  extends BindsAction 
    {
        public CtrlYAction()
        {
           super("CtrlYAction");
        }
        public void actionPerformed(ActionEvent e)
        {
            final SourceToolBar tool = getTools(e);
            if (tool == null)
                return;
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    tool.redo();
                }
            });
        }
    }

    static class CtrlAAction  extends BindsAction 
    {
        public CtrlAAction()
        {
           super("CtrlAAction");
        }
        public void actionPerformed(ActionEvent e)
        {
            final SourceToolBar tool = getTools(e);
            if (tool == null)
                return;
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    tool.setSelectionAll();
                }
            });
        }
    }

    static class CtrlShiftDAction  extends BindsAction 
    {
        public CtrlShiftDAction()
        {
           super("CtrlShiftDAction");
        }
        public void actionPerformed(ActionEvent e)
        {
            final SourceToolBar tool = getTools(e);
            if (tool == null)
                return;
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    tool.lineEndDel();
                }
            });
        }
    }

    static class TabKeyAction  extends BindsAction 
    {
        public TabKeyAction()
        {
            super("TabKeyAction");
        }
        public void actionPerformed(ActionEvent e)
        {
            final SourceToolBar tool = getTools(e);
            if (tool == null)
                return;
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    tool.tabInput();
                }
            });
        }
    }

    static class ShiftTabKeyAction  extends BindsAction 
    {
        public ShiftTabKeyAction()
        {
           super("ShiftTabKeyAction");
        }
        public void actionPerformed(ActionEvent e)
        {
            final SourceToolBar tool = getTools(e);
            if (tool == null)
                return;
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    tool.indentDelTab();
                }
            });
        }
    }

    static class FocusIncTextAction  extends BindsAction 
    {
        public FocusIncTextAction()
        {
           super("FocusIncTextAction");
        }
        public void actionPerformed(ActionEvent e)
        {
            final SourceToolBar tool = getTools(e);
            if (tool == null)
                return;
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    tool.focusedIncText();
                }
            });
        }
    }
}

class BindsAction  extends TextAction 
{
    public BindsAction(String k)
    {
        super(k);
    }

    public void actionPerformed(ActionEvent e)
    {
    }

    public SourceToolBar getTools(ActionEvent e)
    {
        JTextComponent target = getTextComponent(e);
        if (target == null)
            return null;

        if ((! target.isEditable()) || (! target.isEnabled())) 
        {
            UIManager.getLookAndFeel().provideErrorFeedback(target);
            return null;
        }
        return SourceToolBar.toolTbl.get(target);
    }
}


