//============================================================================
//
//    Java Text Editor library
//
//    CustomEditorPane.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.plaf.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.beans.*;

public class CustomEditorPane extends JTextPane 
{
    private boolean isOvertypeMode;
    private final Caret defaultCaret;
    private final Caret overtypeCaret;

    public CustomEditorPane()
    {
        super();

        defaultCaret = new DefaultCaret()
        {
            protected synchronized void damage(Rectangle r)
            {
                if (r != null)
                {
                    JTextComponent c = getComponent();
                    x = 0;
                    y = r.y;
                    width  = c.getSize().width;
                    height = r.height;
                    repaint();
                }
            }
        };
        setBackground(Color.white);
        defaultCaret.setBlinkRate(getCaret().getBlinkRate());
        setCaret(defaultCaret);

        overtypeCaret = new OvertypeCaret();
        overtypeCaret.setBlinkRate(defaultCaret.getBlinkRate());
        setOvertypeMode(false);
    }

    public boolean isOvertypeMode()
    {
        return isOvertypeMode;
    }

    private final Color linecolor = Color.blue;
    private final Color errorcolor= Color.red;
    private final Color elsecolor= Color.gray;

    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D)g;
        Insets i = getInsets();
        int h = g2.getFontMetrics().getHeight();
        int lno = getLineAtCaret(this);
        int y = (getLineAtCaret(this)-1)*h+i.top;

        Paint bk = g2.getPaint();

        if (false)
        {
            g2.setPaint(errorcolor);
            g2.drawLine(i.left, y+h-2, getSize().width-i.left-i.right, y+h-2 );
        }
        g2.setPaint(bk);

    }
    
    /////////////////////////////////////////////////////////////
    
    public void setOvertypeMode(boolean isOvertypeMode)
    {
        this.isOvertypeMode = isOvertypeMode;
        int pos = getCaretPosition();
        if(isOvertypeMode()) 
        {
            setCaret(overtypeCaret);
        }
        else
        {
            setCaret(defaultCaret);
        }
        setCaretPosition(pos);
    }

    public static int getLineAtCaret(JTextComponent component)
    {
        int caretPosition = component.getCaretPosition();
        Element root = component.getDocument().getDefaultRootElement();
        return root.getElementIndex(caretPosition)+1;
    }

    public void replaceSelection(String text)
    {
        if(isOvertypeMode())
        {
            int pos = getCaretPosition();
            if(getSelectedText()==null &&  pos<getDocument().getLength())
            {
                moveCaretPosition(pos+1);
            }
        }
        super.replaceSelection(text);
    }

    protected void processKeyEvent(KeyEvent e)
    {
        super.processKeyEvent(e);
        if(e.getID()==KeyEvent.KEY_RELEASED
          &&  (e.getKeyCode()==KeyEvent.VK_INSERT && e.getModifiers()== 0))
        {
            setCaretPosition(getCaretPosition());
            moveCaretPosition(getCaretPosition());
            setOvertypeMode(!isOvertypeMode());
            repaint();
        }
    }

    ////////////////////////////////////////////////////////////////////

    class OvertypeCaret extends DefaultCaret
    {
        public void paint(Graphics g)
        {
            if(isVisible())
            {
                try
                {
                    JTextComponent component = getComponent();
                    TextUI mapper = component.getUI();
                    Rectangle r = mapper.modelToView(component, getDot());
                    g.setColor(component.getCaretColor());
                    int width = g.getFontMetrics().charWidth('w');

                    if(isOvertypeMode())
                    {
                        int pos = getCaretPosition();
                        if(pos<getDocument().getLength())
                        {
                            if(getSelectedText()!=null)
                            {
                                width = 0;
                            }
                            else
                            {
                                String str = getText(pos, 1);
                                width = g.getFontMetrics().stringWidth(str);
                            }
                        }
                    }

                    int y = r.y + r.height - 2;
                    g.drawLine(r.x, y, r.x + width - 2, y);
                }
                catch(BadLocationException e) 
                { 
                    e.printStackTrace();
                }
            }
        }

        protected synchronized void damage(Rectangle r)
        {
            if (r != null)
            {
                JTextComponent component = getComponent();
                x = r.x;
                y = r.y;
                width = component.getFontMetrics(component.getFont()).charWidth('あ');
                height = r.height;
                repaint();
            }
        }
    }
}

