//============================================================================
//
//    Java Text Editor library
//
//    LNView.java
//
//  Copyright (c) 2013 Fumiaki Hosaka.
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//============================================================================
package text.editor;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.beans.*;

import java.awt.image.*;
import java.awt.geom.*;

public class LNView extends JComponent
{
    private static final int MARGIN = 5;
    private JEditorPane text;
    private FontMetrics fontMetrics;
    private int topInset;
    private int fontAscent;
    private int fontHeight;

    public LNView(JEditorPane textArea)
    {
        text = textArea;
        Font font   = text.getFont();
        this.setFont(font);   
        fontMetrics = text.getFontMetrics(font);
        fontHeight  = fontMetrics.getHeight();
        fontAscent  = fontMetrics.getAscent();
        topInset    = text.getInsets().top;

        text.getDocument().addDocumentListener(new DocumentListener()
        {
            public void insertUpdate(DocumentEvent e)
            {
                repaint();
            }
            public void removeUpdate(DocumentEvent e)
            {
                repaint();
            }
            public void changedUpdate(DocumentEvent e) {}
        });
        
        text.addComponentListener(new ComponentAdapter() 
        {
            public void componentResized(ComponentEvent e)
            {
                revalidate();
                repaint();
            }
        });

        setBorder(BorderFactory.createMatteBorder(0, 0, 0, 1, Color.GRAY));
        setOpaque(true);
        setBackground(Color.white);
    }

    public void resetFont(final Font font)
    {
        fontMetrics = getFontMetrics(font);
        fontHeight  = fontMetrics.getHeight();
        fontAscent  = fontMetrics.getAscent();
        topInset    = text.getInsets().top;
        setFont(font);
     }

    private int getComponentWidth() 
    {
        Document doc  = text.getDocument();
        Element root  = doc.getDefaultRootElement();
        int lineCount = root.getElementIndex(doc.getLength());
        int maxDigits  =  Math.max(3, String.valueOf(lineCount).length());
        return maxDigits*fontMetrics.stringWidth("0")+MARGIN*2;
    }

    public int getLineAtPoint(int y) 
    {
        Element root = text.getDocument().getDefaultRootElement();
        int pos = text.viewToModel(new Point(0, y));
        return root.getElementIndex(pos);
    }

    public Dimension getPreferredSize() 
    {
        return new Dimension(getComponentWidth(), text.getHeight());
    }

    public void paintComponent(Graphics g) 
    {
        boolean hi = false;

        Color bk =  g.getColor();
        Rectangle clip = g.getClipBounds();
        g.setColor(getBackground());
        g.fillRect(clip.x, clip.y, clip.width, clip.height);
        g.setColor(getForeground());
        int base  = clip.y - (topInset);
        int start = getLineAtPoint(base);
        int end   = getLineAtPoint(base+clip.height);
        int y = (topInset)-fontHeight+fontAscent+start*fontHeight;

        int h = fontHeight;
        int cw= getComponentWidth();
        for(int i=start;i<=end;i++) 
        {
            String text = String.valueOf(i+1);
            int w = fontMetrics.stringWidth(text);
            int x = cw - MARGIN - w ;
            int yo = y;
            y = y + fontHeight;

            g.setColor(getForeground());
            g.drawString(text, x, y);
        }
        g.setColor(bk);
    }
}

